/* readonlycollection.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from readonlycollection.vala, do not modify */

/* readonlycollection.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 * Copyright (C) 2010-2014  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;
typedef struct _GeeReadOnlyCollectionPrivate GeeReadOnlyCollectionPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR (gee_read_only_collection_iterator_get_type ())
#define GEE_READ_ONLY_COLLECTION_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR))
#define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorClass))

typedef struct _GeeReadOnlyCollectionIterator GeeReadOnlyCollectionIterator;
typedef struct _GeeReadOnlyCollectionIteratorClass GeeReadOnlyCollectionIteratorClass;
typedef struct _GeeReadOnlyCollectionIteratorPrivate GeeReadOnlyCollectionIteratorPrivate;

typedef gboolean (*GeeForallFunc) (gpointer g, void* user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy** lazy, void* user_data);
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, void* user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, void* user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, void* user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, void* user_data);
struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, void* f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, int* result_length1);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeReadOnlyCollection {
	GObject parent_instance;
	GeeReadOnlyCollectionPrivate * priv;
	GeeCollection* _collection;
};

struct _GeeReadOnlyCollectionClass {
	GObjectClass parent_class;
	GeeCollection* (*get_read_only_view) (GeeReadOnlyCollection* self);
};

struct _GeeReadOnlyCollectionPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};

struct _GeeReadOnlyCollectionIterator {
	GObject parent_instance;
	GeeReadOnlyCollectionIteratorPrivate * priv;
	GeeIterator* _iter;
};

struct _GeeReadOnlyCollectionIteratorClass {
	GObjectClass parent_class;
};

struct _GeeReadOnlyCollectionIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
};


static gpointer gee_read_only_collection_parent_class = NULL;
static gpointer gee_read_only_collection_iterator_parent_class = NULL;
static GeeTraversableIface* gee_read_only_collection_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_read_only_collection_iterator_gee_iterator_parent_iface = NULL;
static GeeTraversableIface* gee_read_only_collection_gee_traversable_parent_iface = NULL;
static GeeIterableIface* gee_read_only_collection_gee_iterable_parent_iface = NULL;
static GeeCollectionIface* gee_read_only_collection_gee_collection_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_lazy (GValue* value, gpointer v_object);
void gee_value_take_lazy (GValue* value, gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
#define GEE_READ_ONLY_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionPrivate))
enum  {
	GEE_READ_ONLY_COLLECTION_DUMMY_PROPERTY,
	GEE_READ_ONLY_COLLECTION_G_TYPE,
	GEE_READ_ONLY_COLLECTION_G_DUP_FUNC,
	GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC,
	GEE_READ_ONLY_COLLECTION_SIZE,
	GEE_READ_ONLY_COLLECTION_READ_ONLY,
	GEE_READ_ONLY_COLLECTION_READ_ONLY_VIEW
};
G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection);
static gboolean gee_read_only_collection_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
gboolean gee_traversable_foreach (GeeTraversable* self, GeeForallFunc f, void* f_target);
static GeeIterator* gee_read_only_collection_real_stream (GeeTraversable* base, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
GeeIterator* gee_traversable_stream (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
static GeeIterator* gee_read_only_collection_real_filter (GeeTraversable* base, GeePredicate f, void* f_target, GDestroyNotify f_target_destroy_notify);
GeeIterator* gee_traversable_filter (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
static GeeIterator* gee_read_only_collection_real_chop (GeeTraversable* base, gint offset, gint length);
GeeIterator* gee_traversable_chop (GeeTraversable* self, gint offset, gint length);
static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base);
GeeIterator* gee_iterable_iterator (GeeIterable* self);
G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeIterator* iterator);
G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeIterator* iterator);
GType gee_read_only_collection_iterator_get_type (void) G_GNUC_CONST;
static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item);
gboolean gee_collection_contains (GeeCollection* self, gconstpointer item);
static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item);
static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item);
static void gee_read_only_collection_real_clear (GeeCollection* base);
static gboolean gee_read_only_collection_real_add_all (GeeCollection* base, GeeCollection* collection);
static gboolean gee_read_only_collection_real_contains_all (GeeCollection* base, GeeCollection* collection);
gboolean gee_collection_contains_all (GeeCollection* self, GeeCollection* collection);
static gboolean gee_read_only_collection_real_remove_all (GeeCollection* base, GeeCollection* collection);
static gboolean gee_read_only_collection_real_retain_all (GeeCollection* base, GeeCollection* collection);
static gpointer* gee_read_only_collection_real_to_array (GeeCollection* base, int* result_length1);
gpointer* gee_collection_to_array (GeeCollection* self, int* result_length1);
gint gee_collection_get_size (GeeCollection* self);
gboolean gee_collection_get_is_empty (GeeCollection* self);
G_GNUC_INTERNAL GeeCollection* gee_read_only_collection_get_read_only_view (GeeReadOnlyCollection* self);
#define GEE_READ_ONLY_COLLECTION_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIteratorPrivate))
enum  {
	GEE_READ_ONLY_COLLECTION_ITERATOR_DUMMY_PROPERTY,
	GEE_READ_ONLY_COLLECTION_ITERATOR_G_TYPE,
	GEE_READ_ONLY_COLLECTION_ITERATOR_G_DUP_FUNC,
	GEE_READ_ONLY_COLLECTION_ITERATOR_G_DESTROY_FUNC,
	GEE_READ_ONLY_COLLECTION_ITERATOR_VALID,
	GEE_READ_ONLY_COLLECTION_ITERATOR_READ_ONLY
};
static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base);
gboolean gee_iterator_next (GeeIterator* self);
static gboolean gee_read_only_collection_iterator_real_has_next (GeeIterator* base);
gboolean gee_iterator_has_next (GeeIterator* self);
static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base);
gpointer gee_iterator_get (GeeIterator* self);
static void gee_read_only_collection_iterator_real_remove (GeeIterator* base);
static gboolean gee_read_only_collection_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
static GeeIterator* gee_read_only_collection_iterator_real_stream (GeeTraversable* base, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
static GeeIterator* gee_read_only_collection_iterator_real_filter (GeeTraversable* base, GeePredicate f, void* f_target, GDestroyNotify f_target_destroy_notify);
static GeeIterator* gee_read_only_collection_iterator_real_chop (GeeTraversable* base, gint offset, gint length);
static GeeIterator** gee_read_only_collection_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1);
GeeIterator** gee_traversable_tee (GeeTraversable* self, guint forks, int* result_length1);
gboolean gee_iterator_get_valid (GeeIterator* self);
static void gee_read_only_collection_iterator_finalize (GObject* obj);
gboolean gee_iterator_get_read_only (GeeIterator* self);
static void _vala_gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void gee_read_only_collection_finalize (GObject* obj);
gboolean gee_collection_get_read_only (GeeCollection* self);
static void _vala_gee_read_only_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_read_only_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);


/**
 * Constructs a read-only collection that mirrors the content of the
 * specified collection.
 *
 * @param collection the collection to decorate.
 */
static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection) {
	GeeReadOnlyCollection * self = NULL;
	GeeCollection* _tmp0_ = NULL;
	GeeCollection* _tmp1_ = NULL;
	g_return_val_if_fail (collection != NULL, NULL);
	self = (GeeReadOnlyCollection*) g_object_new (object_type, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = collection;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_collection);
	self->_collection = _tmp1_;
	return self;
}


G_GNUC_INTERNAL GeeReadOnlyCollection* gee_read_only_collection_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeCollection* collection) {
	return gee_read_only_collection_construct (GEE_TYPE_READ_ONLY_COLLECTION, g_type, g_dup_func, g_destroy_func, collection);
}


/**
 * {@inheritDoc}
 */
static gboolean gee_read_only_collection_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	GeeCollection* _tmp0_ = NULL;
	GeeForallFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp2_ = gee_traversable_foreach ((GeeTraversable*) _tmp0_, _tmp1_, _tmp1__target);
	result = _tmp2_;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeIterator* gee_read_only_collection_real_stream (GeeTraversable* base, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify) {
	GeeReadOnlyCollection * self;
	GeeIterator* result = NULL;
	GeeCollection* _tmp0_ = NULL;
	GeeStreamFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GDestroyNotify _tmp1__target_destroy_notify = NULL;
	GeeIterator* _tmp2_ = NULL;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_stream ((GeeTraversable*) _tmp0_, a_type, (GBoxedCopyFunc) a_dup_func, a_destroy_func, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeIterator* gee_read_only_collection_real_filter (GeeTraversable* base, GeePredicate f, void* f_target, GDestroyNotify f_target_destroy_notify) {
	GeeReadOnlyCollection * self;
	GeeIterator* result = NULL;
	GeeCollection* _tmp0_ = NULL;
	GeePredicate _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GDestroyNotify _tmp1__target_destroy_notify = NULL;
	GeeIterator* _tmp2_ = NULL;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_filter ((GeeTraversable*) _tmp0_, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeIterator* gee_read_only_collection_real_chop (GeeTraversable* base, gint offset, gint length) {
	GeeReadOnlyCollection * self;
	GeeIterator* result = NULL;
	GeeCollection* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	GeeIterator* _tmp3_ = NULL;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = offset;
	_tmp2_ = length;
	_tmp3_ = gee_traversable_chop ((GeeTraversable*) _tmp0_, _tmp1_, _tmp2_);
	result = _tmp3_;
	return result;
}


/**
 * {@inheritDoc}
 */
static GeeIterator* gee_read_only_collection_real_iterator (GeeIterable* base) {
	GeeReadOnlyCollection * self;
	GeeIterator* result = NULL;
	GeeCollection* _tmp0_ = NULL;
	GeeIterator* _tmp1_ = NULL;
	GeeIterator* _tmp2_ = NULL;
	GeeReadOnlyCollectionIterator* _tmp3_ = NULL;
	GeeIterator* _tmp4_ = NULL;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = gee_iterable_iterator ((GeeIterable*) _tmp0_);
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, _tmp2_);
	_tmp4_ = (GeeIterator*) _tmp3_;
	_g_object_unref0 (_tmp2_);
	result = _tmp4_;
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_read_only_collection_real_contains (GeeCollection* base, gconstpointer item) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	GeeCollection* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = item;
	_tmp2_ = gee_collection_contains (_tmp0_, _tmp1_);
	result = _tmp2_;
	return result;
}


/**
 * Unimplemented method (read only collection).
 */
static gboolean gee_read_only_collection_real_add (GeeCollection* base, gconstpointer item) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_assert_not_reached ();
	return result;
}


/**
 * Unimplemented method (read only collection).
 */
static gboolean gee_read_only_collection_real_remove (GeeCollection* base, gconstpointer item) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_assert_not_reached ();
	return result;
}


/**
 * Unimplemented method (read only collection).
 */
static void gee_read_only_collection_real_clear (GeeCollection* base) {
	GeeReadOnlyCollection * self;
	self = (GeeReadOnlyCollection*) base;
	g_assert_not_reached ();
}


/**
 * Unimplemented method (read only collection).
 */
static gboolean gee_read_only_collection_real_add_all (GeeCollection* base, GeeCollection* collection) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_return_val_if_fail (collection != NULL, FALSE);
	g_assert_not_reached ();
	return result;
}


/**
 * {@inheritDoc}
 */
static gboolean gee_read_only_collection_real_contains_all (GeeCollection* base, GeeCollection* collection) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	GeeCollection* _tmp0_ = NULL;
	GeeCollection* _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_return_val_if_fail (collection != NULL, FALSE);
	_tmp0_ = self->_collection;
	_tmp1_ = collection;
	_tmp2_ = gee_collection_contains_all (_tmp0_, _tmp1_);
	result = _tmp2_;
	return result;
}


/**
 * Unimplemented method (read only collection).
 */
static gboolean gee_read_only_collection_real_remove_all (GeeCollection* base, GeeCollection* collection) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_return_val_if_fail (collection != NULL, FALSE);
	g_assert_not_reached ();
	return result;
}


/**
 * Unimplemented method (read only collection).
 */
static gboolean gee_read_only_collection_real_retain_all (GeeCollection* base, GeeCollection* collection) {
	GeeReadOnlyCollection * self;
	gboolean result = FALSE;
	self = (GeeReadOnlyCollection*) base;
	g_return_val_if_fail (collection != NULL, FALSE);
	g_assert_not_reached ();
	return result;
}


/**
 * {@inheritDoc}
 */
static gpointer* gee_read_only_collection_real_to_array (GeeCollection* base, int* result_length1) {
	GeeReadOnlyCollection * self;
	gpointer* result = NULL;
	GeeCollection* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gpointer* _tmp2_ = NULL;
	gpointer* _tmp3_ = NULL;
	gint _tmp3__length1 = 0;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp2_ = gee_collection_to_array (_tmp0_, &_tmp1_);
	_tmp3_ = _tmp2_;
	_tmp3__length1 = _tmp1_;
	if (result_length1) {
		*result_length1 = _tmp3__length1;
	}
	result = _tmp3_;
	return result;
}


static gint gee_read_only_collection_real_get_size (GeeCollection* base) {
	gint result;
	GeeReadOnlyCollection* self;
	GeeCollection* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = gee_collection_get_size (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_read_only_collection_real_get_is_empty (GeeCollection* base) {
	gboolean result;
	GeeReadOnlyCollection* self;
	GeeCollection* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollection*) base;
	_tmp0_ = self->_collection;
	_tmp1_ = gee_collection_get_is_empty (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_read_only_collection_real_get_read_only (GeeCollection* base) {
	gboolean result;
	GeeReadOnlyCollection* self;
	self = (GeeReadOnlyCollection*) base;
	result = TRUE;
	return result;
}


static GType gee_read_only_collection_real_get_element_type (GeeTraversable* base) {
	GType result;
	GeeReadOnlyCollection* self;
	self = (GeeReadOnlyCollection*) base;
	result = self->priv->g_type;
	return result;
}


G_GNUC_INTERNAL GeeCollection* gee_read_only_collection_get_read_only_view (GeeReadOnlyCollection* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_READ_ONLY_COLLECTION_GET_CLASS (self)->get_read_only_view (self);
}


static GeeCollection* gee_read_only_collection_real_get_read_only_view (GeeReadOnlyCollection* base) {
	GeeCollection* result;
	GeeReadOnlyCollection* self;
	GeeCollection* _tmp0_ = NULL;
	self = base;
	_tmp0_ = _g_object_ref0 ((GeeCollection*) self);
	result = _tmp0_;
	return result;
}


G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeIterator* iterator) {
	GeeReadOnlyCollectionIterator * self = NULL;
	GeeIterator* _tmp0_ = NULL;
	GeeIterator* _tmp1_ = NULL;
	g_return_val_if_fail (iterator != NULL, NULL);
	self = (GeeReadOnlyCollectionIterator*) g_object_new (object_type, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = iterator;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_iter);
	self->_iter = _tmp1_;
	return self;
}


G_GNUC_INTERNAL GeeReadOnlyCollectionIterator* gee_read_only_collection_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeIterator* iterator) {
	return gee_read_only_collection_iterator_construct (GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, iterator);
}


static gboolean gee_read_only_collection_iterator_real_next (GeeIterator* base) {
	GeeReadOnlyCollectionIterator * self;
	gboolean result = FALSE;
	GeeIterator* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_iterator_next (_tmp0_);
	result = _tmp1_;
	return result;
}


static gboolean gee_read_only_collection_iterator_real_has_next (GeeIterator* base) {
	GeeReadOnlyCollectionIterator * self;
	gboolean result = FALSE;
	GeeIterator* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_iterator_has_next (_tmp0_);
	result = _tmp1_;
	return result;
}


static gpointer gee_read_only_collection_iterator_real_get (GeeIterator* base) {
	GeeReadOnlyCollectionIterator * self;
	gpointer result = NULL;
	GeeIterator* _tmp0_ = NULL;
	gpointer _tmp1_ = NULL;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_iterator_get (_tmp0_);
	result = _tmp1_;
	return result;
}


static void gee_read_only_collection_iterator_real_remove (GeeIterator* base) {
	GeeReadOnlyCollectionIterator * self;
	self = (GeeReadOnlyCollectionIterator*) base;
	g_assert_not_reached ();
}


static gboolean gee_read_only_collection_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeReadOnlyCollectionIterator * self;
	gboolean result = FALSE;
	GeeIterator* _tmp0_ = NULL;
	GeeForallFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp2_ = gee_traversable_foreach ((GeeTraversable*) _tmp0_, _tmp1_, _tmp1__target);
	result = _tmp2_;
	return result;
}


static GeeIterator* gee_read_only_collection_iterator_real_stream (GeeTraversable* base, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify) {
	GeeReadOnlyCollectionIterator * self;
	GeeIterator* result = NULL;
	GeeIterator* _tmp0_ = NULL;
	GeeStreamFunc _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GDestroyNotify _tmp1__target_destroy_notify = NULL;
	GeeIterator* _tmp2_ = NULL;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_stream ((GeeTraversable*) _tmp0_, a_type, (GBoxedCopyFunc) a_dup_func, a_destroy_func, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}


static GeeIterator* gee_read_only_collection_iterator_real_filter (GeeTraversable* base, GeePredicate f, void* f_target, GDestroyNotify f_target_destroy_notify) {
	GeeReadOnlyCollectionIterator * self;
	GeeIterator* result = NULL;
	GeeIterator* _tmp0_ = NULL;
	GeePredicate _tmp1_ = NULL;
	void* _tmp1__target = NULL;
	GDestroyNotify _tmp1__target_destroy_notify = NULL;
	GeeIterator* _tmp2_ = NULL;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = f;
	_tmp1__target = f_target;
	_tmp1__target_destroy_notify = f_target_destroy_notify;
	f_target_destroy_notify = NULL;
	_tmp2_ = gee_traversable_filter ((GeeTraversable*) _tmp0_, _tmp1_, _tmp1__target, _tmp1__target_destroy_notify);
	result = _tmp2_;
	(f_target_destroy_notify == NULL) ? NULL : (f_target_destroy_notify (f_target), NULL);
	f = NULL;
	f_target = NULL;
	f_target_destroy_notify = NULL;
	return result;
}


static GeeIterator* gee_read_only_collection_iterator_real_chop (GeeTraversable* base, gint offset, gint length) {
	GeeReadOnlyCollectionIterator * self;
	GeeIterator* result = NULL;
	GeeIterator* _tmp0_ = NULL;
	gint _tmp1_ = 0;
	gint _tmp2_ = 0;
	GeeIterator* _tmp3_ = NULL;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = offset;
	_tmp2_ = length;
	_tmp3_ = gee_traversable_chop ((GeeTraversable*) _tmp0_, _tmp1_, _tmp2_);
	result = _tmp3_;
	return result;
}


static GeeIterator** gee_read_only_collection_iterator_real_tee (GeeTraversable* base, guint forks, int* result_length1) {
	GeeReadOnlyCollectionIterator * self;
	GeeIterator** result = NULL;
	guint _tmp0_ = 0U;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = forks;
	if (_tmp0_ == ((guint) 0)) {
		GeeIterator** _tmp1_ = NULL;
		GeeIterator** _tmp2_ = NULL;
		gint _tmp2__length1 = 0;
		_tmp1_ = g_new0 (GeeIterator*, 0 + 1);
		_tmp2_ = _tmp1_;
		_tmp2__length1 = 0;
		if (result_length1) {
			*result_length1 = _tmp2__length1;
		}
		result = _tmp2_;
		return result;
	} else {
		GeeIterator** iters = NULL;
		GeeIterator* _tmp3_ = NULL;
		guint _tmp4_ = 0U;
		gint _tmp5_ = 0;
		GeeIterator** _tmp6_ = NULL;
		gint iters_length1 = 0;
		gint _iters_size_ = 0;
		GeeIterator** _result_ = NULL;
		guint _tmp7_ = 0U;
		GeeIterator** _tmp8_ = NULL;
		gint _result__length1 = 0;
		gint __result__size_ = 0;
		GeeIterator** _tmp9_ = NULL;
		gint _tmp9__length1 = 0;
		GeeIterator* _tmp10_ = NULL;
		GeeIterator* _tmp11_ = NULL;
		GeeIterator** _tmp31_ = NULL;
		gint _tmp31__length1 = 0;
		_tmp3_ = self->_iter;
		_tmp4_ = forks;
		_tmp6_ = gee_traversable_tee ((GeeTraversable*) _tmp3_, _tmp4_, &_tmp5_);
		iters = _tmp6_;
		iters_length1 = _tmp5_;
		_iters_size_ = iters_length1;
		_tmp7_ = forks;
		_tmp8_ = g_new0 (GeeIterator*, _tmp7_ + 1);
		_result_ = _tmp8_;
		_result__length1 = _tmp7_;
		__result__size_ = _result__length1;
		_tmp9_ = iters;
		_tmp9__length1 = iters_length1;
		_tmp10_ = _tmp9_[0];
		_tmp11_ = self->_iter;
		if (_tmp10_ == _tmp11_) {
			GeeIterator** _tmp12_ = NULL;
			gint _tmp12__length1 = 0;
			GeeIterator* _tmp13_ = NULL;
			GeeIterator* _tmp14_ = NULL;
			_tmp12_ = _result_;
			_tmp12__length1 = _result__length1;
			_tmp13_ = _g_object_ref0 ((GeeIterator*) self);
			_g_object_unref0 (_tmp12_[0]);
			_tmp12_[0] = _tmp13_;
			_tmp14_ = _tmp12_[0];
		} else {
			GeeIterator** _tmp15_ = NULL;
			gint _tmp15__length1 = 0;
			GeeIterator** _tmp16_ = NULL;
			gint _tmp16__length1 = 0;
			GeeIterator* _tmp17_ = NULL;
			GeeReadOnlyCollectionIterator* _tmp18_ = NULL;
			GeeIterator* _tmp19_ = NULL;
			_tmp15_ = _result_;
			_tmp15__length1 = _result__length1;
			_tmp16_ = iters;
			_tmp16__length1 = iters_length1;
			_tmp17_ = _tmp16_[0];
			_tmp18_ = gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, _tmp17_);
			_g_object_unref0 (_tmp15_[0]);
			_tmp15_[0] = (GeeIterator*) _tmp18_;
			_tmp19_ = _tmp15_[0];
		}
		{
			guint i = 0U;
			i = (guint) 1;
			{
				gboolean _tmp20_ = FALSE;
				_tmp20_ = TRUE;
				while (TRUE) {
					guint _tmp22_ = 0U;
					guint _tmp23_ = 0U;
					GeeIterator** _tmp24_ = NULL;
					gint _tmp24__length1 = 0;
					guint _tmp25_ = 0U;
					GeeIterator** _tmp26_ = NULL;
					gint _tmp26__length1 = 0;
					guint _tmp27_ = 0U;
					GeeIterator* _tmp28_ = NULL;
					GeeReadOnlyCollectionIterator* _tmp29_ = NULL;
					GeeIterator* _tmp30_ = NULL;
					if (!_tmp20_) {
						guint _tmp21_ = 0U;
						_tmp21_ = i;
						i = _tmp21_ + 1;
					}
					_tmp20_ = FALSE;
					_tmp22_ = i;
					_tmp23_ = forks;
					if (!(_tmp22_ < _tmp23_)) {
						break;
					}
					_tmp24_ = _result_;
					_tmp24__length1 = _result__length1;
					_tmp25_ = i;
					_tmp26_ = iters;
					_tmp26__length1 = iters_length1;
					_tmp27_ = i;
					_tmp28_ = _tmp26_[_tmp27_];
					_tmp29_ = gee_read_only_collection_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, _tmp28_);
					_g_object_unref0 (_tmp24_[_tmp25_]);
					_tmp24_[_tmp25_] = (GeeIterator*) _tmp29_;
					_tmp30_ = _tmp24_[_tmp25_];
				}
			}
		}
		_tmp31_ = _result_;
		_tmp31__length1 = _result__length1;
		if (result_length1) {
			*result_length1 = _tmp31__length1;
		}
		result = _tmp31_;
		iters = (_vala_array_free (iters, iters_length1, (GDestroyNotify) g_object_unref), NULL);
		return result;
	}
}


static gboolean gee_read_only_collection_iterator_real_get_valid (GeeIterator* base) {
	gboolean result;
	GeeReadOnlyCollectionIterator* self;
	GeeIterator* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	self = (GeeReadOnlyCollectionIterator*) base;
	_tmp0_ = self->_iter;
	_tmp1_ = gee_iterator_get_valid (_tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_;
	return result;
}


static gboolean gee_read_only_collection_iterator_real_get_read_only (GeeIterator* base) {
	gboolean result;
	GeeReadOnlyCollectionIterator* self;
	self = (GeeReadOnlyCollectionIterator*) base;
	result = TRUE;
	return result;
}


static GType gee_read_only_collection_iterator_real_get_element_type (GeeTraversable* base) {
	GType result;
	GeeReadOnlyCollectionIterator* self;
	self = (GeeReadOnlyCollectionIterator*) base;
	result = self->priv->g_type;
	return result;
}


static void gee_read_only_collection_iterator_class_init (GeeReadOnlyCollectionIteratorClass * klass) {
	gee_read_only_collection_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeReadOnlyCollectionIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_collection_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_collection_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_read_only_collection_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ITERATOR_VALID, g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_ITERATOR_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_read_only_collection_iterator_gee_traversable_get_g_type (GeeReadOnlyCollectionIterator* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_read_only_collection_iterator_gee_traversable_get_g_dup_func (GeeReadOnlyCollectionIterator* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_read_only_collection_iterator_gee_traversable_get_g_destroy_func (GeeReadOnlyCollectionIterator* self) {
	return self->priv->g_destroy_func;
}


static void gee_read_only_collection_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_read_only_collection_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_read_only_collection_iterator_real_foreach;
	iface->stream = (GeeIterator* (*)(GeeTraversable*, GType, GBoxedCopyFunc, GDestroyNotify, GeeStreamFunc, void*, GDestroyNotify)) gee_read_only_collection_iterator_real_stream;
	iface->filter = (GeeIterator* (*)(GeeTraversable*, GeePredicate, void*, GDestroyNotify)) gee_read_only_collection_iterator_real_filter;
	iface->chop = (GeeIterator* (*)(GeeTraversable*, gint, gint)) gee_read_only_collection_iterator_real_chop;
	iface->tee = (GeeIterator** (*)(GeeTraversable*, guint, int*)) gee_read_only_collection_iterator_real_tee;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_read_only_collection_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_read_only_collection_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_read_only_collection_iterator_gee_traversable_get_g_destroy_func;
	iface->get_element_type = gee_read_only_collection_iterator_real_get_element_type;
}


static void gee_read_only_collection_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_read_only_collection_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = (gboolean (*)(GeeIterator*)) gee_read_only_collection_iterator_real_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_read_only_collection_iterator_real_has_next;
	iface->get = (gpointer (*)(GeeIterator*)) gee_read_only_collection_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_read_only_collection_iterator_real_remove;
	iface->get_valid = gee_read_only_collection_iterator_real_get_valid;
	iface->get_read_only = gee_read_only_collection_iterator_real_get_read_only;
}


static void gee_read_only_collection_iterator_instance_init (GeeReadOnlyCollectionIterator * self) {
	self->priv = GEE_READ_ONLY_COLLECTION_ITERATOR_GET_PRIVATE (self);
}


static void gee_read_only_collection_iterator_finalize (GObject* obj) {
	GeeReadOnlyCollectionIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator);
	_g_object_unref0 (self->_iter);
	G_OBJECT_CLASS (gee_read_only_collection_iterator_parent_class)->finalize (obj);
}


GType gee_read_only_collection_iterator_get_type (void) {
	static volatile gsize gee_read_only_collection_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_read_only_collection_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyCollectionIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_collection_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyCollectionIterator), 0, (GInstanceInitFunc) gee_read_only_collection_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_read_only_collection_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_read_only_collection_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_read_only_collection_iterator_type_id;
		gee_read_only_collection_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeReadOnlyCollectionIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_read_only_collection_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_read_only_collection_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_once_init_leave (&gee_read_only_collection_iterator_type_id__volatile, gee_read_only_collection_iterator_type_id);
	}
	return gee_read_only_collection_iterator_type_id__volatile;
}


static void _vala_gee_read_only_collection_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeReadOnlyCollectionIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator);
	switch (property_id) {
		case GEE_READ_ONLY_COLLECTION_ITERATOR_VALID:
		g_value_set_boolean (value, gee_iterator_get_valid ((GeeIterator*) self));
		break;
		case GEE_READ_ONLY_COLLECTION_ITERATOR_READ_ONLY:
		g_value_set_boolean (value, gee_iterator_get_read_only ((GeeIterator*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_read_only_collection_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeReadOnlyCollectionIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_READ_ONLY_COLLECTION_TYPE_ITERATOR, GeeReadOnlyCollectionIterator);
	switch (property_id) {
		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_COLLECTION_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_read_only_collection_class_init (GeeReadOnlyCollectionClass * klass) {
	gee_read_only_collection_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeReadOnlyCollectionPrivate));
	GEE_READ_ONLY_COLLECTION_CLASS (klass)->get_read_only_view = gee_read_only_collection_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_read_only_collection_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_read_only_collection_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_read_only_collection_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	/**
	 * {@inheritDoc}
	 */
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_READ_ONLY_COLLECTION_READ_ONLY_VIEW, g_param_spec_object ("read-only-view", "read-only-view", "read-only-view", GEE_TYPE_COLLECTION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_read_only_collection_gee_traversable_get_g_type (GeeReadOnlyCollection* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_read_only_collection_gee_traversable_get_g_dup_func (GeeReadOnlyCollection* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_read_only_collection_gee_traversable_get_g_destroy_func (GeeReadOnlyCollection* self) {
	return self->priv->g_destroy_func;
}


static void gee_read_only_collection_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_read_only_collection_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_read_only_collection_real_foreach;
	iface->stream = (GeeIterator* (*)(GeeTraversable*, GType, GBoxedCopyFunc, GDestroyNotify, GeeStreamFunc, void*, GDestroyNotify)) gee_read_only_collection_real_stream;
	iface->filter = (GeeIterator* (*)(GeeTraversable*, GeePredicate, void*, GDestroyNotify)) gee_read_only_collection_real_filter;
	iface->chop = (GeeIterator* (*)(GeeTraversable*, gint, gint)) gee_read_only_collection_real_chop;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_read_only_collection_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_read_only_collection_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_read_only_collection_gee_traversable_get_g_destroy_func;
	iface->get_element_type = gee_read_only_collection_real_get_element_type;
}


static GType gee_read_only_collection_gee_iterable_get_g_type (GeeReadOnlyCollection* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_read_only_collection_gee_iterable_get_g_dup_func (GeeReadOnlyCollection* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_read_only_collection_gee_iterable_get_g_destroy_func (GeeReadOnlyCollection* self) {
	return self->priv->g_destroy_func;
}


static void gee_read_only_collection_gee_iterable_interface_init (GeeIterableIface * iface) {
	gee_read_only_collection_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->iterator = (GeeIterator* (*)(GeeIterable*)) gee_read_only_collection_real_iterator;
	iface->get_g_type = (GType(*)(GeeIterable*)) gee_read_only_collection_gee_iterable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeIterable*)) gee_read_only_collection_gee_iterable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeIterable*)) gee_read_only_collection_gee_iterable_get_g_destroy_func;
}


static GType gee_read_only_collection_gee_collection_get_g_type (GeeReadOnlyCollection* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_read_only_collection_gee_collection_get_g_dup_func (GeeReadOnlyCollection* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_read_only_collection_gee_collection_get_g_destroy_func (GeeReadOnlyCollection* self) {
	return self->priv->g_destroy_func;
}


static void gee_read_only_collection_gee_collection_interface_init (GeeCollectionIface * iface) {
	gee_read_only_collection_gee_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->contains = (gboolean (*)(GeeCollection*, gconstpointer)) gee_read_only_collection_real_contains;
	iface->add = (gboolean (*)(GeeCollection*, gconstpointer)) gee_read_only_collection_real_add;
	iface->remove = (gboolean (*)(GeeCollection*, gconstpointer)) gee_read_only_collection_real_remove;
	iface->clear = (void (*)(GeeCollection*)) gee_read_only_collection_real_clear;
	iface->add_all = (gboolean (*)(GeeCollection*, GeeCollection*)) gee_read_only_collection_real_add_all;
	iface->contains_all = (gboolean (*)(GeeCollection*, GeeCollection*)) gee_read_only_collection_real_contains_all;
	iface->remove_all = (gboolean (*)(GeeCollection*, GeeCollection*)) gee_read_only_collection_real_remove_all;
	iface->retain_all = (gboolean (*)(GeeCollection*, GeeCollection*)) gee_read_only_collection_real_retain_all;
	iface->to_array = (gpointer* (*)(GeeCollection*, int*)) gee_read_only_collection_real_to_array;
	iface->get_g_type = (GType(*)(GeeCollection*)) gee_read_only_collection_gee_collection_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeCollection*)) gee_read_only_collection_gee_collection_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeCollection*)) gee_read_only_collection_gee_collection_get_g_destroy_func;
	iface->get_size = gee_read_only_collection_real_get_size;
	iface->get_is_empty = gee_read_only_collection_real_get_is_empty;
	iface->get_read_only = gee_read_only_collection_real_get_read_only;
	iface->get_read_only_view = (GeeCollection* (*) (GeeCollection *)) gee_read_only_collection_get_read_only_view;
}


static void gee_read_only_collection_instance_init (GeeReadOnlyCollection * self) {
	self->priv = GEE_READ_ONLY_COLLECTION_GET_PRIVATE (self);
}


static void gee_read_only_collection_finalize (GObject* obj) {
	GeeReadOnlyCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection);
	_g_object_unref0 (self->_collection);
	G_OBJECT_CLASS (gee_read_only_collection_parent_class)->finalize (obj);
}


/**
 * Read-only view for {@link Collection} collections.
 *
 * This class decorates any class which implements the {@link Collection}
 * interface by making it read only. Any method which normally modify data will
 * throw an error.
 *
 * @see Collection
 */
G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) {
	static volatile gsize gee_read_only_collection_type_id__volatile = 0;
	if (g_once_init_enter (&gee_read_only_collection_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeReadOnlyCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_read_only_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeReadOnlyCollection), 0, (GInstanceInitFunc) gee_read_only_collection_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_read_only_collection_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_read_only_collection_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_collection_info = { (GInterfaceInitFunc) gee_read_only_collection_gee_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_read_only_collection_type_id;
		gee_read_only_collection_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeReadOnlyCollection", &g_define_type_info, 0);
		g_type_add_interface_static (gee_read_only_collection_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_read_only_collection_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (gee_read_only_collection_type_id, GEE_TYPE_COLLECTION, &gee_collection_info);
		g_once_init_leave (&gee_read_only_collection_type_id__volatile, gee_read_only_collection_type_id);
	}
	return gee_read_only_collection_type_id__volatile;
}


static void _vala_gee_read_only_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeReadOnlyCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection);
	switch (property_id) {
		case GEE_READ_ONLY_COLLECTION_SIZE:
		g_value_set_int (value, gee_collection_get_size ((GeeCollection*) self));
		break;
		case GEE_READ_ONLY_COLLECTION_READ_ONLY:
		g_value_set_boolean (value, gee_collection_get_read_only ((GeeCollection*) self));
		break;
		case GEE_READ_ONLY_COLLECTION_READ_ONLY_VIEW:
		g_value_take_object (value, gee_read_only_collection_get_read_only_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_read_only_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeReadOnlyCollection * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection);
	switch (property_id) {
		case GEE_READ_ONLY_COLLECTION_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_READ_ONLY_COLLECTION_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_READ_ONLY_COLLECTION_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}



