/* comparable.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from comparable.vala, do not modify */

/* comparable.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_COMPARABLE (gee_comparable_get_type ())
#define GEE_COMPARABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COMPARABLE, GeeComparable))
#define GEE_IS_COMPARABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COMPARABLE))
#define GEE_COMPARABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COMPARABLE, GeeComparableIface))

typedef struct _GeeComparable GeeComparable;
typedef struct _GeeComparableIface GeeComparableIface;

struct _GeeComparableIface {
	GTypeInterface parent_iface;
	gint (*compare_to) (GeeComparable* self, gconstpointer object);
};



GType gee_comparable_get_type (void) G_GNUC_CONST;
gint gee_comparable_compare_to (GeeComparable* self, gconstpointer object);


/**
 * Compares this object with the specifed object.
 *
 * @return a negative integer, zero, or a positive integer as this object
 *         is less than, equal to, or greater than the specified object
 */
gint gee_comparable_compare_to (GeeComparable* self, gconstpointer object) {
	g_return_val_if_fail (self != NULL, 0);
	return GEE_COMPARABLE_GET_INTERFACE (self)->compare_to (self, object);
}


static void gee_comparable_base_init (GeeComparableIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


/**
 * This interface defines a total ordering among instances of each class
 * implementing it.
 *
 * In other words:
 *
 *   * It's irreflexive: For all `a` it holds that `a.compare_to(a) == 0`
 *   * It's transitive: For all `a`, `b` and `c` if `a.compare_to(b) < 0` and
 *     `b.compare_to(c) < 0` then `a.compare_to(c) < 0`.
 *   * It's trichotomous: For all `a` and `b` it holds that
 *     `a.compare_to(b) = -b.compare_to(a)`.
 *
 * Note: The relationship must be immutable. In other words if at one point of
 *   program `a.compare_to(b)` had certain value then call `a.compare_to(b)`
 *   //must always// return the original value until end of `a` and `b` lifetime.
 *
 * @see Hashable
 */
GType gee_comparable_get_type (void) {
	static volatile gsize gee_comparable_type_id__volatile = 0;
	if (g_once_init_enter (&gee_comparable_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeComparableIface), (GBaseInitFunc) gee_comparable_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType gee_comparable_type_id;
		gee_comparable_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeComparable", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gee_comparable_type_id, G_TYPE_OBJECT);
		g_once_init_leave (&gee_comparable_type_id__volatile, gee_comparable_type_id);
	}
	return gee_comparable_type_id__volatile;
}



