/* bidirlist.vala
 *
 * Copyright (C) 2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */
[GenericAccessors]
public interface Gee.BidirList<G> : Gee.List<G> {
	/**
	 * Returns a BidirListIterator that can be used for iteration over this list.
	 *
	 * @return a BidirListIterator that can be used for iteration over this list
	 */
	public abstract new BidirListIterator<G> bidir_list_iterator ();

	/**
	 * The read-only view of this list.
	 */
	public abstract new BidirList<G> read_only_view { owned get; }
}
