/* abstractmultiset.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from abstractmultiset.vala, do not modify */

/* abstractmultiset.vala
 *
 * Copyright (C) 2009  Ali Sabil
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Ali Sabil <ali.sabil@gmail.com>
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_TRAVERSABLE (gee_traversable_get_type ())
#define GEE_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversable))
#define GEE_IS_TRAVERSABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TRAVERSABLE))
#define GEE_TRAVERSABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_TRAVERSABLE, GeeTraversableIface))

typedef struct _GeeTraversable GeeTraversable;
typedef struct _GeeTraversableIface GeeTraversableIface;

#define GEE_TRAVERSABLE_TYPE_STREAM (gee_traversable_stream_get_type ())

#define GEE_TYPE_LAZY (gee_lazy_get_type ())
#define GEE_LAZY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_LAZY, GeeLazy))
#define GEE_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_LAZY, GeeLazyClass))
#define GEE_IS_LAZY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_LAZY))
#define GEE_IS_LAZY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_LAZY))
#define GEE_LAZY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_LAZY, GeeLazyClass))

typedef struct _GeeLazy GeeLazy;
typedef struct _GeeLazyClass GeeLazyClass;

#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_ITERABLE (gee_iterable_get_type ())
#define GEE_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERABLE, GeeIterable))
#define GEE_IS_ITERABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERABLE))
#define GEE_ITERABLE_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERABLE, GeeIterableIface))

typedef struct _GeeIterable GeeIterable;
typedef struct _GeeIterableIface GeeIterableIface;

#define GEE_TYPE_COLLECTION (gee_collection_get_type ())
#define GEE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_COLLECTION, GeeCollection))
#define GEE_IS_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_COLLECTION))
#define GEE_COLLECTION_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_COLLECTION, GeeCollectionIface))

typedef struct _GeeCollection GeeCollection;
typedef struct _GeeCollectionIface GeeCollectionIface;

#define GEE_TYPE_ABSTRACT_COLLECTION (gee_abstract_collection_get_type ())
#define GEE_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollection))
#define GEE_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))
#define GEE_IS_ABSTRACT_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_IS_ABSTRACT_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_COLLECTION))
#define GEE_ABSTRACT_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_COLLECTION, GeeAbstractCollectionClass))

typedef struct _GeeAbstractCollection GeeAbstractCollection;
typedef struct _GeeAbstractCollectionClass GeeAbstractCollectionClass;
typedef struct _GeeAbstractCollectionPrivate GeeAbstractCollectionPrivate;

#define GEE_TYPE_MULTI_SET (gee_multi_set_get_type ())
#define GEE_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MULTI_SET, GeeMultiSet))
#define GEE_IS_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MULTI_SET))
#define GEE_MULTI_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MULTI_SET, GeeMultiSetIface))

typedef struct _GeeMultiSet GeeMultiSet;
typedef struct _GeeMultiSetIface GeeMultiSetIface;

#define GEE_TYPE_ABSTRACT_MULTI_SET (gee_abstract_multi_set_get_type ())
#define GEE_ABSTRACT_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet))
#define GEE_ABSTRACT_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSetClass))
#define GEE_IS_ABSTRACT_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ABSTRACT_MULTI_SET))
#define GEE_IS_ABSTRACT_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_ABSTRACT_MULTI_SET))
#define GEE_ABSTRACT_MULTI_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSetClass))

typedef struct _GeeAbstractMultiSet GeeAbstractMultiSet;
typedef struct _GeeAbstractMultiSetClass GeeAbstractMultiSetClass;
typedef struct _GeeAbstractMultiSetPrivate GeeAbstractMultiSetPrivate;

#define GEE_TYPE_MAP (gee_map_get_type ())
#define GEE_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP, GeeMap))
#define GEE_IS_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP))
#define GEE_MAP_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP, GeeMapIface))

typedef struct _GeeMap GeeMap;
typedef struct _GeeMapIface GeeMapIface;

#define GEE_TYPE_MAP_ITERATOR (gee_map_iterator_get_type ())
#define GEE_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIterator))
#define GEE_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP_ITERATOR))
#define GEE_MAP_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIteratorIface))

typedef struct _GeeMapIterator GeeMapIterator;
typedef struct _GeeMapIteratorIface GeeMapIteratorIface;

#define GEE_TYPE_SET (gee_set_get_type ())
#define GEE_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SET, GeeSet))
#define GEE_IS_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SET))
#define GEE_SET_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_SET, GeeSetIface))

typedef struct _GeeSet GeeSet;
typedef struct _GeeSetIface GeeSetIface;

#define GEE_MAP_TYPE_ENTRY (gee_map_entry_get_type ())
#define GEE_MAP_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntry))
#define GEE_MAP_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))
#define GEE_MAP_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_IS_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_MAP_TYPE_ENTRY))
#define GEE_MAP_ENTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_MAP_TYPE_ENTRY, GeeMapEntryClass))

typedef struct _GeeMapEntry GeeMapEntry;
typedef struct _GeeMapEntryClass GeeMapEntryClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR (gee_abstract_multi_set_iterator_get_type ())
#define GEE_ABSTRACT_MULTI_SET_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator))
#define GEE_ABSTRACT_MULTI_SET_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIteratorClass))
#define GEE_ABSTRACT_MULTI_SET_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR))
#define GEE_ABSTRACT_MULTI_SET_IS_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR))
#define GEE_ABSTRACT_MULTI_SET_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIteratorClass))

typedef struct _GeeAbstractMultiSetIterator GeeAbstractMultiSetIterator;
typedef struct _GeeAbstractMultiSetIteratorClass GeeAbstractMultiSetIteratorClass;

#define GEE_TYPE_READ_ONLY_COLLECTION (gee_read_only_collection_get_type ())
#define GEE_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollection))
#define GEE_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))
#define GEE_IS_READ_ONLY_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_IS_READ_ONLY_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_COLLECTION))
#define GEE_READ_ONLY_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_COLLECTION, GeeReadOnlyCollectionClass))

typedef struct _GeeReadOnlyCollection GeeReadOnlyCollection;
typedef struct _GeeReadOnlyCollectionClass GeeReadOnlyCollectionClass;

#define GEE_TYPE_READ_ONLY_MULTI_SET (gee_read_only_multi_set_get_type ())
#define GEE_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSet))
#define GEE_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))
#define GEE_IS_READ_ONLY_MULTI_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_IS_READ_ONLY_MULTI_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_READ_ONLY_MULTI_SET))
#define GEE_READ_ONLY_MULTI_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_READ_ONLY_MULTI_SET, GeeReadOnlyMultiSetClass))

typedef struct _GeeReadOnlyMultiSet GeeReadOnlyMultiSet;
typedef struct _GeeReadOnlyMultiSetClass GeeReadOnlyMultiSetClass;
typedef struct _GeeAbstractMultiSetIteratorPrivate GeeAbstractMultiSetIteratorPrivate;
#define _vala_assert(expr, msg) if G_LIKELY (expr) ; else g_assertion_message_expr (G_LOG_DOMAIN, __FILE__, __LINE__, G_STRFUNC, msg);

typedef gboolean (*GeeForallFunc) (gpointer g, void* user_data);
typedef enum  {
	GEE_TRAVERSABLE_STREAM_YIELD,
	GEE_TRAVERSABLE_STREAM_CONTINUE,
	GEE_TRAVERSABLE_STREAM_END,
	GEE_TRAVERSABLE_STREAM_WAIT
} GeeTraversableStream;

typedef GeeTraversableStream (*GeeStreamFunc) (GeeTraversableStream state, GeeLazy* g, GeeLazy** lazy, void* user_data);
struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
	gboolean (*get_valid) (GeeIterator* self);
	gboolean (*get_read_only) (GeeIterator* self);
};

typedef gpointer (*GeeFoldFunc) (gpointer g, gpointer a, void* user_data);
typedef gpointer (*GeeMapFunc) (gpointer g, void* user_data);
typedef gboolean (*GeePredicate) (gconstpointer g, void* user_data);
typedef GeeIterator* (*GeeFlatMapFunc) (gpointer g, void* user_data);
struct _GeeTraversableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeTraversable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeTraversable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeTraversable* self);
	gboolean (*foreach) (GeeTraversable* self, GeeForallFunc f, void* f_target);
	GeeIterator* (*stream) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeStreamFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	gpointer (*fold) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeMapFunc f, void* f_target);
	GeeIterator* (*scan) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldFunc f, void* f_target, gpointer seed);
	GeeIterator* (*filter) (GeeTraversable* self, GeePredicate pred, void* pred_target, GDestroyNotify pred_target_destroy_notify);
	GeeIterator* (*chop) (GeeTraversable* self, gint offset, gint length);
	GType (*get_element_type) (GeeTraversable* self);
	GeeIterator* (*flat_map) (GeeTraversable* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFlatMapFunc f, void* f_target, GDestroyNotify f_target_destroy_notify);
	GeeIterator** (*tee) (GeeTraversable* self, guint forks, int* result_length1);
};

struct _GeeIterableIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeIterable* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeIterable* self);
	GDestroyNotify (*get_g_destroy_func) (GeeIterable* self);
	GeeIterator* (*iterator) (GeeIterable* self);
};

struct _GeeCollectionIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeCollection* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeCollection* self);
	GDestroyNotify (*get_g_destroy_func) (GeeCollection* self);
	gboolean (*contains) (GeeCollection* self, gconstpointer item);
	gboolean (*add) (GeeCollection* self, gconstpointer item);
	gboolean (*remove) (GeeCollection* self, gconstpointer item);
	void (*clear) (GeeCollection* self);
	gboolean (*add_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*contains_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*remove_all) (GeeCollection* self, GeeCollection* collection);
	gboolean (*retain_all) (GeeCollection* self, GeeCollection* collection);
	gpointer* (*to_array) (GeeCollection* self, int* result_length1);
	gint (*get_size) (GeeCollection* self);
	gboolean (*get_is_empty) (GeeCollection* self);
	gboolean (*get_read_only) (GeeCollection* self);
	GeeCollection* (*get_read_only_view) (GeeCollection* self);
	gboolean (*add_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*contains_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*remove_all_array) (GeeCollection* self, gpointer* array, int array_length1);
	gboolean (*add_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*contains_all_iterator) (GeeCollection* self, GeeIterator* iter);
	gboolean (*remove_all_iterator) (GeeCollection* self, GeeIterator* iter);
};

struct _GeeAbstractCollection {
	GObject parent_instance;
	GeeAbstractCollectionPrivate * priv;
};

struct _GeeAbstractCollectionClass {
	GObjectClass parent_class;
	gboolean (*contains) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*add) (GeeAbstractCollection* self, gconstpointer item);
	gboolean (*remove) (GeeAbstractCollection* self, gconstpointer item);
	void (*clear) (GeeAbstractCollection* self);
	GeeIterator* (*iterator) (GeeAbstractCollection* self);
	gboolean (*foreach) (GeeAbstractCollection* self, GeeForallFunc f, void* f_target);
	void (*reserved0) (GeeAbstractCollection* self);
	void (*reserved1) (GeeAbstractCollection* self);
	void (*reserved2) (GeeAbstractCollection* self);
	void (*reserved3) (GeeAbstractCollection* self);
	void (*reserved4) (GeeAbstractCollection* self);
	void (*reserved5) (GeeAbstractCollection* self);
	void (*reserved6) (GeeAbstractCollection* self);
	void (*reserved7) (GeeAbstractCollection* self);
	void (*reserved8) (GeeAbstractCollection* self);
	void (*reserved9) (GeeAbstractCollection* self);
	gint (*get_size) (GeeAbstractCollection* self);
	gboolean (*get_read_only) (GeeAbstractCollection* self);
	GeeCollection* (*get_read_only_view) (GeeAbstractCollection* self);
};

struct _GeeMultiSetIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeMultiSet* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeMultiSet* self);
	GDestroyNotify (*get_g_destroy_func) (GeeMultiSet* self);
	gint (*count) (GeeMultiSet* self, gconstpointer item);
	GeeMultiSet* (*get_read_only_view) (GeeMultiSet* self);
};

typedef gpointer (*GeeFoldMapFunc) (gconstpointer k, gconstpointer v, gpointer a, void* user_data);
typedef gboolean (*GeeForallMapFunc) (gconstpointer k, gconstpointer v, void* user_data);
struct _GeeMapIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMapIterator* self);
	GType (*get_v_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMapIterator* self);
	gboolean (*next) (GeeMapIterator* self);
	gboolean (*has_next) (GeeMapIterator* self);
	gpointer (*get_key) (GeeMapIterator* self);
	gpointer (*get_value) (GeeMapIterator* self);
	void (*set_value) (GeeMapIterator* self, gconstpointer value);
	void (*unset) (GeeMapIterator* self);
	gpointer (*fold) (GeeMapIterator* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldMapFunc f, void* f_target, gpointer seed);
	gboolean (*foreach) (GeeMapIterator* self, GeeForallMapFunc f, void* f_target);
	gboolean (*get_valid) (GeeMapIterator* self);
	gboolean (*get_mutable) (GeeMapIterator* self);
	gboolean (*get_read_only) (GeeMapIterator* self);
};

struct _GeeSetIface {
	GTypeInterface parent_iface;
	GType (*get_g_type) (GeeSet* self);
	GBoxedCopyFunc (*get_g_dup_func) (GeeSet* self);
	GDestroyNotify (*get_g_destroy_func) (GeeSet* self);
	GeeSet* (*get_read_only_view) (GeeSet* self);
};

struct _GeeMapIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMap* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMap* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMap* self);
	GType (*get_v_type) (GeeMap* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMap* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMap* self);
	gboolean (*has_key) (GeeMap* self, gconstpointer key);
	gboolean (*has) (GeeMap* self, gconstpointer key, gconstpointer value);
	gpointer (*get) (GeeMap* self, gconstpointer key);
	void (*set) (GeeMap* self, gconstpointer key, gconstpointer value);
	gboolean (*unset) (GeeMap* self, gconstpointer key, gpointer* value);
	void (*clear) (GeeMap* self);
	GeeMapIterator* (*map_iterator) (GeeMap* self);
	void (*set_all) (GeeMap* self, GeeMap* map);
	gboolean (*unset_all) (GeeMap* self, GeeMap* map);
	gboolean (*has_all) (GeeMap* self, GeeMap* map);
	gint (*get_size) (GeeMap* self);
	gboolean (*get_is_empty) (GeeMap* self);
	gboolean (*get_read_only) (GeeMap* self);
	GeeSet* (*get_keys) (GeeMap* self);
	GeeCollection* (*get_values) (GeeMap* self);
	GeeSet* (*get_entries) (GeeMap* self);
	GeeMap* (*get_read_only_view) (GeeMap* self);
};

struct _GeeAbstractMultiSet {
	GeeAbstractCollection parent_instance;
	GeeAbstractMultiSetPrivate * priv;
	GeeMap* _storage_map;
};

struct _GeeAbstractMultiSetClass {
	GeeAbstractCollectionClass parent_class;
	void (*reserved0) (GeeAbstractMultiSet* self);
	void (*reserved1) (GeeAbstractMultiSet* self);
	void (*reserved2) (GeeAbstractMultiSet* self);
	void (*reserved3) (GeeAbstractMultiSet* self);
	void (*reserved4) (GeeAbstractMultiSet* self);
	void (*reserved5) (GeeAbstractMultiSet* self);
	void (*reserved6) (GeeAbstractMultiSet* self);
	void (*reserved7) (GeeAbstractMultiSet* self);
	void (*reserved8) (GeeAbstractMultiSet* self);
	GeeMultiSet* (*get_read_only_view) (GeeAbstractMultiSet* self);
};

struct _GeeAbstractMultiSetPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	gint _nitems;
	GWeakRef _read_only_view;
};

struct _GeeAbstractMultiSetIterator {
	GObject parent_instance;
	GeeAbstractMultiSetIteratorPrivate * priv;
};

struct _GeeAbstractMultiSetIteratorClass {
	GObjectClass parent_class;
};

struct _GeeAbstractMultiSetIteratorPrivate {
	GType g_type;
	GBoxedCopyFunc g_dup_func;
	GDestroyNotify g_destroy_func;
	GeeAbstractMultiSet* _set;
	GeeMapIterator* _iter;
	gint _pending;
	gboolean _removed;
};


static gpointer gee_abstract_multi_set_parent_class = NULL;
static gpointer gee_abstract_multi_set_iterator_parent_class = NULL;
static GeeTraversableIface* gee_abstract_multi_set_iterator_gee_traversable_parent_iface = NULL;
static GeeIteratorIface* gee_abstract_multi_set_iterator_gee_iterator_parent_iface = NULL;
static GeeMultiSetIface* gee_abstract_multi_set_gee_multi_set_parent_iface = NULL;

GType gee_traversable_stream_get_type (void) G_GNUC_CONST;
gpointer gee_lazy_ref (gpointer instance);
void gee_lazy_unref (gpointer instance);
GParamSpec* gee_param_spec_lazy (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gee_value_set_lazy (GValue* value, gpointer v_object);
void gee_value_take_lazy (GValue* value, gpointer v_object);
gpointer gee_value_get_lazy (const GValue* value);
GType gee_lazy_get_type (void) G_GNUC_CONST;
GType gee_iterator_get_type (void) G_GNUC_CONST;
GType gee_traversable_get_type (void) G_GNUC_CONST;
GType gee_iterable_get_type (void) G_GNUC_CONST;
GType gee_collection_get_type (void) G_GNUC_CONST;
GType gee_abstract_collection_get_type (void) G_GNUC_CONST;
GType gee_multi_set_get_type (void) G_GNUC_CONST;
GType gee_abstract_multi_set_get_type (void) G_GNUC_CONST;
GType gee_map_iterator_get_type (void) G_GNUC_CONST;
GType gee_set_get_type (void) G_GNUC_CONST;
GType gee_map_entry_get_type (void) G_GNUC_CONST;
GType gee_map_get_type (void) G_GNUC_CONST;
#define GEE_ABSTRACT_MULTI_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSetPrivate))
enum  {
	GEE_ABSTRACT_MULTI_SET_DUMMY_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_G_TYPE,
	GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC,
	GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC,
	GEE_ABSTRACT_MULTI_SET_SIZE,
	GEE_ABSTRACT_MULTI_SET_READ_ONLY
};
GeeAbstractMultiSet* gee_abstract_multi_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeMap* storage_map);
GeeAbstractCollection* gee_abstract_collection_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func);
static gint gee_abstract_multi_set_real_count (GeeMultiSet* base, gconstpointer item);
gboolean gee_map_has_key (GeeMap* self, gconstpointer key);
gpointer gee_map_get (GeeMap* self, gconstpointer key);
static gboolean gee_abstract_multi_set_real_contains (GeeAbstractCollection* base, gconstpointer item);
static GeeIterator* gee_abstract_multi_set_real_iterator (GeeAbstractCollection* base);
static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeAbstractMultiSet* set);
static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeAbstractMultiSet* set);
static GType gee_abstract_multi_set_iterator_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static gboolean gee_abstract_multi_set_real_add (GeeAbstractCollection* base, gconstpointer item);
void gee_map_set (GeeMap* self, gconstpointer key, gconstpointer value);
static gboolean gee_abstract_multi_set_real_remove (GeeAbstractCollection* base, gconstpointer item);
gboolean gee_map_unset (GeeMap* self, gconstpointer key, gpointer* value);
static void gee_abstract_multi_set_real_clear (GeeAbstractCollection* base);
void gee_map_clear (GeeMap* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved0 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved0 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved1 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved1 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved2 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved2 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved3 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved3 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved4 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved4 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved5 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved5 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved6 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved6 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved7 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved7 (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL void gee_abstract_multi_set_reserved8 (GeeAbstractMultiSet* self);
static void gee_abstract_multi_set_real_reserved8 (GeeAbstractMultiSet* self);
GeeMultiSet* gee_abstract_multi_set_get_read_only_view (GeeAbstractMultiSet* self);
G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeMultiSet* multiset);
G_GNUC_INTERNAL GeeReadOnlyMultiSet* gee_read_only_multi_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeMultiSet* multiset);
G_GNUC_INTERNAL GType gee_read_only_collection_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
G_GNUC_INTERNAL GType gee_read_only_multi_set_get_type (void) G_GNUC_CONST G_GNUC_UNUSED;
static GObject * gee_abstract_multi_set_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
#define GEE_ABSTRACT_MULTI_SET_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIteratorPrivate))
enum  {
	GEE_ABSTRACT_MULTI_SET_ITERATOR_DUMMY_PROPERTY,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY,
	GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID
};
GeeMapIterator* gee_map_map_iterator (GeeMap* self);
static gboolean gee_abstract_multi_set_iterator_real_next (GeeIterator* base);
gboolean gee_map_iterator_next (GeeMapIterator* self);
gpointer gee_map_iterator_get_value (GeeMapIterator* self);
static gboolean gee_abstract_multi_set_iterator_real_has_next (GeeIterator* base);
gboolean gee_map_iterator_has_next (GeeMapIterator* self);
static gpointer gee_abstract_multi_set_iterator_real_get (GeeIterator* base);
gpointer gee_map_iterator_get_key (GeeMapIterator* self);
static void gee_abstract_multi_set_iterator_real_remove (GeeIterator* base);
void gee_map_iterator_set_value (GeeMapIterator* self, gconstpointer value);
void gee_map_iterator_unset (GeeMapIterator* self);
static gboolean gee_abstract_multi_set_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target);
gboolean gee_map_iterator_get_valid (GeeMapIterator* self);
static void gee_abstract_multi_set_iterator_finalize (GObject* obj);
gboolean gee_iterator_get_read_only (GeeIterator* self);
gboolean gee_iterator_get_valid (GeeIterator* self);
static void _vala_gee_abstract_multi_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_abstract_multi_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void gee_abstract_multi_set_finalize (GObject* obj);
gint gee_abstract_collection_get_size (GeeAbstractCollection* self);
gboolean gee_abstract_collection_get_read_only (GeeAbstractCollection* self);
static void _vala_gee_abstract_multi_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_gee_abstract_multi_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);


/**
 * Constructs a new, empty abstract multi set.
 */
static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


GeeAbstractMultiSet* gee_abstract_multi_set_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeMap* storage_map) {
	GeeAbstractMultiSet * self = NULL;
	GeeMap* _tmp0_ = NULL;
	GeeMap* _tmp1_ = NULL;
	g_return_val_if_fail (storage_map != NULL, NULL);
	self = (GeeAbstractMultiSet*) gee_abstract_collection_construct (object_type, g_type, (GBoxedCopyFunc) g_dup_func, g_destroy_func);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = storage_map;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->_storage_map);
	self->_storage_map = _tmp1_;
	return self;
}


static gint gee_abstract_multi_set_real_count (GeeMultiSet* base, gconstpointer item) {
	GeeAbstractMultiSet * self;
	gint result = 0;
	gint _result_ = 0;
	GeeMap* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeAbstractMultiSet*) base;
	_result_ = 0;
	_tmp0_ = self->_storage_map;
	_tmp1_ = item;
	_tmp2_ = gee_map_has_key (_tmp0_, _tmp1_);
	if (_tmp2_) {
		GeeMap* _tmp3_ = NULL;
		gconstpointer _tmp4_ = NULL;
		gpointer _tmp5_ = NULL;
		_tmp3_ = self->_storage_map;
		_tmp4_ = item;
		_tmp5_ = gee_map_get (_tmp3_, _tmp4_);
		_result_ = (gint) ((gintptr) _tmp5_);
	}
	result = _result_;
	return result;
}


static gboolean gee_abstract_multi_set_real_contains (GeeAbstractCollection* base, gconstpointer item) {
	GeeAbstractMultiSet * self;
	gboolean result = FALSE;
	GeeMap* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	_tmp1_ = item;
	_tmp2_ = gee_map_has_key (_tmp0_, _tmp1_);
	result = _tmp2_;
	return result;
}


static GeeIterator* gee_abstract_multi_set_real_iterator (GeeAbstractCollection* base) {
	GeeAbstractMultiSet * self;
	GeeIterator* result = NULL;
	GeeAbstractMultiSetIterator* _tmp0_ = NULL;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = gee_abstract_multi_set_iterator_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, self);
	result = (GeeIterator*) _tmp0_;
	return result;
}


static gboolean gee_abstract_multi_set_real_add (GeeAbstractCollection* base, gconstpointer item) {
	GeeAbstractMultiSet * self;
	gboolean result = FALSE;
	GeeMap* _tmp0_ = NULL;
	gconstpointer _tmp1_ = NULL;
	gboolean _tmp2_ = FALSE;
	gint _tmp11_ = 0;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	_tmp1_ = item;
	_tmp2_ = gee_map_has_key (_tmp0_, _tmp1_);
	if (_tmp2_) {
		gint current_count = 0;
		GeeMap* _tmp3_ = NULL;
		gconstpointer _tmp4_ = NULL;
		gpointer _tmp5_ = NULL;
		GeeMap* _tmp6_ = NULL;
		gconstpointer _tmp7_ = NULL;
		gint _tmp8_ = 0;
		_tmp3_ = self->_storage_map;
		_tmp4_ = item;
		_tmp5_ = gee_map_get (_tmp3_, _tmp4_);
		current_count = (gint) ((gintptr) _tmp5_);
		_tmp6_ = self->_storage_map;
		_tmp7_ = item;
		_tmp8_ = current_count;
		gee_map_set (_tmp6_, _tmp7_, (gpointer) ((gintptr) (_tmp8_ + 1)));
	} else {
		GeeMap* _tmp9_ = NULL;
		gconstpointer _tmp10_ = NULL;
		_tmp9_ = self->_storage_map;
		_tmp10_ = item;
		gee_map_set (_tmp9_, _tmp10_, (gpointer) ((gintptr) 1));
	}
	_tmp11_ = self->priv->_nitems;
	self->priv->_nitems = _tmp11_ + 1;
	result = TRUE;
	return result;
}


static gboolean gee_abstract_multi_set_real_remove (GeeAbstractCollection* base, gconstpointer item) {
	GeeAbstractMultiSet * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gint _tmp1_ = 0;
	self = (GeeAbstractMultiSet*) base;
	_tmp1_ = self->priv->_nitems;
	if (_tmp1_ > 0) {
		GeeMap* _tmp2_ = NULL;
		gconstpointer _tmp3_ = NULL;
		gboolean _tmp4_ = FALSE;
		_tmp2_ = self->_storage_map;
		_tmp3_ = item;
		_tmp4_ = gee_map_has_key (_tmp2_, _tmp3_);
		_tmp0_ = _tmp4_;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gint current_count = 0;
		GeeMap* _tmp5_ = NULL;
		gconstpointer _tmp6_ = NULL;
		gpointer _tmp7_ = NULL;
		gint _tmp8_ = 0;
		gint _tmp14_ = 0;
		_tmp5_ = self->_storage_map;
		_tmp6_ = item;
		_tmp7_ = gee_map_get (_tmp5_, _tmp6_);
		current_count = (gint) ((gintptr) _tmp7_);
		_tmp8_ = current_count;
		if (_tmp8_ <= 1) {
			GeeMap* _tmp9_ = NULL;
			gconstpointer _tmp10_ = NULL;
			_tmp9_ = self->_storage_map;
			_tmp10_ = item;
			gee_map_unset (_tmp9_, _tmp10_, NULL);
		} else {
			GeeMap* _tmp11_ = NULL;
			gconstpointer _tmp12_ = NULL;
			gint _tmp13_ = 0;
			_tmp11_ = self->_storage_map;
			_tmp12_ = item;
			_tmp13_ = current_count;
			gee_map_set (_tmp11_, _tmp12_, (gpointer) ((gintptr) (_tmp13_ - 1)));
		}
		_tmp14_ = self->priv->_nitems;
		self->priv->_nitems = _tmp14_ - 1;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


static void gee_abstract_multi_set_real_clear (GeeAbstractCollection* base) {
	GeeAbstractMultiSet * self;
	GeeMap* _tmp0_ = NULL;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->_storage_map;
	gee_map_clear (_tmp0_);
	self->priv->_nitems = 0;
}


static void gee_abstract_multi_set_real_reserved0 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved0 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved0 (self);
}


static void gee_abstract_multi_set_real_reserved1 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved1 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved1 (self);
}


static void gee_abstract_multi_set_real_reserved2 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved2 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved2 (self);
}


static void gee_abstract_multi_set_real_reserved3 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved3 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved3 (self);
}


static void gee_abstract_multi_set_real_reserved4 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved4 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved4 (self);
}


static void gee_abstract_multi_set_real_reserved5 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved5 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved5 (self);
}


static void gee_abstract_multi_set_real_reserved6 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved6 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved6 (self);
}


static void gee_abstract_multi_set_real_reserved7 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved7 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved7 (self);
}


static void gee_abstract_multi_set_real_reserved8 (GeeAbstractMultiSet* self) {
}


void gee_abstract_multi_set_reserved8 (GeeAbstractMultiSet* self) {
	g_return_if_fail (self != NULL);
	GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->reserved8 (self);
}


static gint gee_abstract_multi_set_real_get_size (GeeAbstractCollection* base) {
	gint result;
	GeeAbstractMultiSet* self;
	gint _tmp0_ = 0;
	self = (GeeAbstractMultiSet*) base;
	_tmp0_ = self->priv->_nitems;
	result = _tmp0_;
	return result;
}


static gboolean gee_abstract_multi_set_real_get_read_only (GeeAbstractCollection* base) {
	gboolean result;
	GeeAbstractMultiSet* self;
	self = (GeeAbstractMultiSet*) base;
	result = FALSE;
	return result;
}


GeeMultiSet* gee_abstract_multi_set_get_read_only_view (GeeAbstractMultiSet* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return GEE_ABSTRACT_MULTI_SET_GET_CLASS (self)->get_read_only_view (self);
}


static GeeMultiSet* gee_abstract_multi_set_real_get_read_only_view (GeeAbstractMultiSet* base) {
	GeeMultiSet* result;
	GeeAbstractMultiSet* self;
	GeeMultiSet* instance = NULL;
	GObject* _tmp0_ = NULL;
	GeeMultiSet* _tmp1_ = NULL;
	self = base;
	_tmp0_ = g_weak_ref_get (&self->priv->_read_only_view);
	instance = G_TYPE_CHECK_INSTANCE_CAST (_tmp0_, GEE_TYPE_MULTI_SET, GeeMultiSet);
	_tmp1_ = instance;
	if (_tmp1_ == NULL) {
		GeeReadOnlyMultiSet* _tmp2_ = NULL;
		GeeMultiSet* _tmp3_ = NULL;
		_tmp2_ = gee_read_only_multi_set_new (self->priv->g_type, (GBoxedCopyFunc) self->priv->g_dup_func, self->priv->g_destroy_func, (GeeMultiSet*) self);
		_g_object_unref0 (instance);
		instance = (GeeMultiSet*) _tmp2_;
		_tmp3_ = instance;
		g_weak_ref_set (&self->priv->_read_only_view, (GObject*) _tmp3_);
	}
	result = instance;
	return result;
}


static GObject * gee_abstract_multi_set_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GObjectClass * parent_class;
	GeeAbstractMultiSet * self;
	parent_class = G_OBJECT_CLASS (gee_abstract_multi_set_parent_class);
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	g_weak_ref_clear (&self->priv->_read_only_view);
	g_weak_ref_init (&self->priv->_read_only_view, NULL);
	return obj;
}


static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_construct (GType object_type, GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeAbstractMultiSet* set) {
	GeeAbstractMultiSetIterator * self = NULL;
	GeeAbstractMultiSet* _tmp0_ = NULL;
	GeeAbstractMultiSet* _tmp1_ = NULL;
	GeeAbstractMultiSet* _tmp2_ = NULL;
	GeeMap* _tmp3_ = NULL;
	GeeMapIterator* _tmp4_ = NULL;
	g_return_val_if_fail (set != NULL, NULL);
	self = (GeeAbstractMultiSetIterator*) g_object_new (object_type, NULL);
	self->priv->g_type = g_type;
	self->priv->g_dup_func = g_dup_func;
	self->priv->g_destroy_func = g_destroy_func;
	_tmp0_ = set;
	_tmp1_ = _g_object_ref0 (_tmp0_);
	_g_object_unref0 (self->priv->_set);
	self->priv->_set = _tmp1_;
	_tmp2_ = self->priv->_set;
	_tmp3_ = _tmp2_->_storage_map;
	_tmp4_ = gee_map_map_iterator (_tmp3_);
	_g_object_unref0 (self->priv->_iter);
	self->priv->_iter = _tmp4_;
	return self;
}


static GeeAbstractMultiSetIterator* gee_abstract_multi_set_iterator_new (GType g_type, GBoxedCopyFunc g_dup_func, GDestroyNotify g_destroy_func, GeeAbstractMultiSet* set) {
	return gee_abstract_multi_set_iterator_construct (GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, g_type, g_dup_func, g_destroy_func, set);
}


static gboolean gee_abstract_multi_set_iterator_real_next (GeeIterator* base) {
	GeeAbstractMultiSetIterator * self;
	gboolean result = FALSE;
	gint _tmp0_ = 0;
	self = (GeeAbstractMultiSetIterator*) base;
	self->priv->_removed = FALSE;
	_tmp0_ = self->priv->_pending;
	if (_tmp0_ == 0) {
		GeeMapIterator* _tmp1_ = NULL;
		gboolean _tmp2_ = FALSE;
		_tmp1_ = self->priv->_iter;
		_tmp2_ = gee_map_iterator_next (_tmp1_);
		if (_tmp2_) {
			GeeMapIterator* _tmp3_ = NULL;
			gpointer _tmp4_ = NULL;
			_tmp3_ = self->priv->_iter;
			_tmp4_ = gee_map_iterator_get_value (_tmp3_);
			self->priv->_pending = ((gint) ((gintptr) _tmp4_)) - 1;
			result = TRUE;
			return result;
		}
	} else {
		gint _tmp5_ = 0;
		_tmp5_ = self->priv->_pending;
		self->priv->_pending = _tmp5_ - 1;
		result = TRUE;
		return result;
	}
	result = FALSE;
	return result;
}


static gboolean gee_abstract_multi_set_iterator_real_has_next (GeeIterator* base) {
	GeeAbstractMultiSetIterator * self;
	gboolean result = FALSE;
	gboolean _tmp0_ = FALSE;
	gint _tmp1_ = 0;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp1_ = self->priv->_pending;
	if (_tmp1_ > 0) {
		_tmp0_ = TRUE;
	} else {
		GeeMapIterator* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		_tmp2_ = self->priv->_iter;
		_tmp3_ = gee_map_iterator_has_next (_tmp2_);
		_tmp0_ = _tmp3_;
	}
	result = _tmp0_;
	return result;
}


static gpointer gee_abstract_multi_set_iterator_real_get (GeeIterator* base) {
	GeeAbstractMultiSetIterator * self;
	gpointer result = NULL;
	gboolean _tmp0_ = FALSE;
	GeeMapIterator* _tmp1_ = NULL;
	gpointer _tmp2_ = NULL;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp0_ = self->priv->_removed;
	_vala_assert (!_tmp0_, "! _removed");
	_tmp1_ = self->priv->_iter;
	_tmp2_ = gee_map_iterator_get_key (_tmp1_);
	result = _tmp2_;
	return result;
}


static void gee_abstract_multi_set_iterator_real_remove (GeeIterator* base) {
	GeeAbstractMultiSetIterator * self;
	gboolean _tmp0_ = FALSE;
	GeeMapIterator* _tmp1_ = NULL;
	GeeMapIterator* _tmp2_ = NULL;
	gpointer _tmp3_ = NULL;
	gint _tmp4_ = 0;
	gint _tmp5_ = 0;
	GeeAbstractMultiSet* _tmp7_ = NULL;
	gint _tmp8_ = 0;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp0_ = self->priv->_removed;
	_vala_assert (!_tmp0_, "! _removed");
	_tmp1_ = self->priv->_iter;
	_tmp2_ = self->priv->_iter;
	_tmp3_ = gee_map_iterator_get_value (_tmp2_);
	self->priv->_pending = ((gint) ((gintptr) _tmp3_)) - 1;
	_tmp4_ = self->priv->_pending;
	gee_map_iterator_set_value (_tmp1_, (gpointer) ((gintptr) _tmp4_));
	_tmp5_ = self->priv->_pending;
	if (_tmp5_ == 0) {
		GeeMapIterator* _tmp6_ = NULL;
		_tmp6_ = self->priv->_iter;
		gee_map_iterator_unset (_tmp6_);
	}
	_tmp7_ = self->priv->_set;
	_tmp8_ = _tmp7_->priv->_nitems;
	_tmp7_->priv->_nitems = _tmp8_ - 1;
	self->priv->_removed = TRUE;
}


static gboolean gee_abstract_multi_set_iterator_real_foreach (GeeTraversable* base, GeeForallFunc f, void* f_target) {
	GeeAbstractMultiSetIterator * self;
	gboolean result = FALSE;
	GeeMapIterator* _tmp0_ = NULL;
	gboolean _tmp1_ = FALSE;
	gboolean _tmp2_ = FALSE;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp0_ = self->priv->_iter;
	_tmp1_ = gee_map_iterator_get_valid (_tmp0_);
	_tmp2_ = _tmp1_;
	if (_tmp2_) {
		gboolean _tmp3_ = FALSE;
		_tmp3_ = self->priv->_removed;
		if (!_tmp3_) {
			GeeForallFunc _tmp4_ = NULL;
			void* _tmp4__target = NULL;
			GeeMapIterator* _tmp5_ = NULL;
			gpointer _tmp6_ = NULL;
			gboolean _tmp7_ = FALSE;
			_tmp4_ = f;
			_tmp4__target = f_target;
			_tmp5_ = self->priv->_iter;
			_tmp6_ = gee_map_iterator_get_key (_tmp5_);
			_tmp7_ = _tmp4_ (_tmp6_, _tmp4__target);
			if (!_tmp7_) {
				result = FALSE;
				return result;
			}
		}
		{
			gint i = 0;
			gint _tmp8_ = 0;
			_tmp8_ = self->priv->_pending;
			i = _tmp8_ - 1;
			{
				gboolean _tmp9_ = FALSE;
				_tmp9_ = TRUE;
				while (TRUE) {
					gint _tmp11_ = 0;
					GeeForallFunc _tmp12_ = NULL;
					void* _tmp12__target = NULL;
					GeeMapIterator* _tmp13_ = NULL;
					gpointer _tmp14_ = NULL;
					gboolean _tmp15_ = FALSE;
					if (!_tmp9_) {
						gint _tmp10_ = 0;
						_tmp10_ = i;
						i = _tmp10_ - 1;
					}
					_tmp9_ = FALSE;
					_tmp11_ = i;
					if (!(_tmp11_ >= 0)) {
						break;
					}
					_tmp12_ = f;
					_tmp12__target = f_target;
					_tmp13_ = self->priv->_iter;
					_tmp14_ = gee_map_iterator_get_key (_tmp13_);
					_tmp15_ = _tmp12_ (_tmp14_, _tmp12__target);
					if (!_tmp15_) {
						gint _tmp16_ = 0;
						_tmp16_ = i;
						self->priv->_pending = _tmp16_;
						result = FALSE;
						return result;
					}
				}
			}
		}
	}
	while (TRUE) {
		GeeMapIterator* _tmp17_ = NULL;
		gboolean _tmp18_ = FALSE;
		_tmp17_ = self->priv->_iter;
		_tmp18_ = gee_map_iterator_next (_tmp17_);
		if (!_tmp18_) {
			break;
		}
		{
			gint i = 0;
			GeeMapIterator* _tmp19_ = NULL;
			gpointer _tmp20_ = NULL;
			_tmp19_ = self->priv->_iter;
			_tmp20_ = gee_map_iterator_get_value (_tmp19_);
			i = ((gint) ((gintptr) _tmp20_)) - 1;
			{
				gboolean _tmp21_ = FALSE;
				_tmp21_ = TRUE;
				while (TRUE) {
					gint _tmp23_ = 0;
					GeeForallFunc _tmp24_ = NULL;
					void* _tmp24__target = NULL;
					GeeMapIterator* _tmp25_ = NULL;
					gpointer _tmp26_ = NULL;
					gboolean _tmp27_ = FALSE;
					if (!_tmp21_) {
						gint _tmp22_ = 0;
						_tmp22_ = i;
						i = _tmp22_ - 1;
					}
					_tmp21_ = FALSE;
					_tmp23_ = i;
					if (!(_tmp23_ >= 0)) {
						break;
					}
					_tmp24_ = f;
					_tmp24__target = f_target;
					_tmp25_ = self->priv->_iter;
					_tmp26_ = gee_map_iterator_get_key (_tmp25_);
					_tmp27_ = _tmp24_ (_tmp26_, _tmp24__target);
					if (!_tmp27_) {
						gint _tmp28_ = 0;
						self->priv->_removed = FALSE;
						_tmp28_ = i;
						self->priv->_pending = _tmp28_;
						result = FALSE;
						return result;
					}
				}
			}
		}
	}
	self->priv->_removed = FALSE;
	self->priv->_pending = 0;
	result = TRUE;
	return result;
}


static gboolean gee_abstract_multi_set_iterator_real_get_read_only (GeeIterator* base) {
	gboolean result;
	GeeAbstractMultiSetIterator* self;
	self = (GeeAbstractMultiSetIterator*) base;
	result = FALSE;
	return result;
}


static gboolean gee_abstract_multi_set_iterator_real_get_valid (GeeIterator* base) {
	gboolean result;
	GeeAbstractMultiSetIterator* self;
	gboolean _tmp0_ = FALSE;
	gboolean _tmp1_ = FALSE;
	self = (GeeAbstractMultiSetIterator*) base;
	_tmp1_ = self->priv->_removed;
	if (!_tmp1_) {
		GeeMapIterator* _tmp2_ = NULL;
		gboolean _tmp3_ = FALSE;
		gboolean _tmp4_ = FALSE;
		_tmp2_ = self->priv->_iter;
		_tmp3_ = gee_map_iterator_get_valid (_tmp2_);
		_tmp4_ = _tmp3_;
		_tmp0_ = _tmp4_;
	} else {
		_tmp0_ = FALSE;
	}
	result = _tmp0_;
	return result;
}


static void gee_abstract_multi_set_iterator_class_init (GeeAbstractMultiSetIteratorClass * klass) {
	gee_abstract_multi_set_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeAbstractMultiSetIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_multi_set_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_multi_set_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_multi_set_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID, g_param_spec_boolean ("valid", "valid", "valid", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_abstract_multi_set_iterator_gee_traversable_get_g_type (GeeAbstractMultiSetIterator* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_abstract_multi_set_iterator_gee_traversable_get_g_dup_func (GeeAbstractMultiSetIterator* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_abstract_multi_set_iterator_gee_traversable_get_g_destroy_func (GeeAbstractMultiSetIterator* self) {
	return self->priv->g_destroy_func;
}


static void gee_abstract_multi_set_iterator_gee_traversable_interface_init (GeeTraversableIface * iface) {
	gee_abstract_multi_set_iterator_gee_traversable_parent_iface = g_type_interface_peek_parent (iface);
	iface->foreach = (gboolean (*)(GeeTraversable*, GeeForallFunc, void*)) gee_abstract_multi_set_iterator_real_foreach;
	iface->get_g_type = (GType(*)(GeeTraversable*)) gee_abstract_multi_set_iterator_gee_traversable_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeTraversable*)) gee_abstract_multi_set_iterator_gee_traversable_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeTraversable*)) gee_abstract_multi_set_iterator_gee_traversable_get_g_destroy_func;
}


static void gee_abstract_multi_set_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_abstract_multi_set_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = (gboolean (*)(GeeIterator*)) gee_abstract_multi_set_iterator_real_next;
	iface->has_next = (gboolean (*)(GeeIterator*)) gee_abstract_multi_set_iterator_real_has_next;
	iface->get = (gpointer (*)(GeeIterator*)) gee_abstract_multi_set_iterator_real_get;
	iface->remove = (void (*)(GeeIterator*)) gee_abstract_multi_set_iterator_real_remove;
	iface->get_read_only = gee_abstract_multi_set_iterator_real_get_read_only;
	iface->get_valid = gee_abstract_multi_set_iterator_real_get_valid;
}


static void gee_abstract_multi_set_iterator_instance_init (GeeAbstractMultiSetIterator * self) {
	self->priv = GEE_ABSTRACT_MULTI_SET_ITERATOR_GET_PRIVATE (self);
	self->priv->_pending = 0;
	self->priv->_removed = FALSE;
}


static void gee_abstract_multi_set_iterator_finalize (GObject* obj) {
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	_g_object_unref0 (self->priv->_set);
	_g_object_unref0 (self->priv->_iter);
	G_OBJECT_CLASS (gee_abstract_multi_set_iterator_parent_class)->finalize (obj);
}


static GType gee_abstract_multi_set_iterator_get_type (void) {
	static volatile gsize gee_abstract_multi_set_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_abstract_multi_set_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractMultiSetIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_multi_set_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractMultiSetIterator), 0, (GInstanceInitFunc) gee_abstract_multi_set_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_traversable_info = { (GInterfaceInitFunc) gee_abstract_multi_set_iterator_gee_traversable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_abstract_multi_set_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_abstract_multi_set_iterator_type_id;
		gee_abstract_multi_set_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeAbstractMultiSetIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_abstract_multi_set_iterator_type_id, GEE_TYPE_TRAVERSABLE, &gee_traversable_info);
		g_type_add_interface_static (gee_abstract_multi_set_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
		g_once_init_leave (&gee_abstract_multi_set_iterator_type_id__volatile, gee_abstract_multi_set_iterator_type_id);
	}
	return gee_abstract_multi_set_iterator_type_id__volatile;
}


static void _vala_gee_abstract_multi_set_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_READ_ONLY:
		g_value_set_boolean (value, gee_iterator_get_read_only ((GeeIterator*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_VALID:
		g_value_set_boolean (value, gee_iterator_get_valid ((GeeIterator*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_abstract_multi_set_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeAbstractMultiSetIterator * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_ABSTRACT_MULTI_SET_TYPE_ITERATOR, GeeAbstractMultiSetIterator);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_ITERATOR_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_abstract_multi_set_class_init (GeeAbstractMultiSetClass * klass) {
	gee_abstract_multi_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeAbstractMultiSetPrivate));
	((GeeAbstractCollectionClass *) klass)->contains = gee_abstract_multi_set_real_contains;
	((GeeAbstractCollectionClass *) klass)->iterator = gee_abstract_multi_set_real_iterator;
	((GeeAbstractCollectionClass *) klass)->add = gee_abstract_multi_set_real_add;
	((GeeAbstractCollectionClass *) klass)->remove = gee_abstract_multi_set_real_remove;
	((GeeAbstractCollectionClass *) klass)->clear = gee_abstract_multi_set_real_clear;
	((GeeAbstractMultiSetClass *) klass)->reserved0 = gee_abstract_multi_set_real_reserved0;
	((GeeAbstractMultiSetClass *) klass)->reserved1 = gee_abstract_multi_set_real_reserved1;
	((GeeAbstractMultiSetClass *) klass)->reserved2 = gee_abstract_multi_set_real_reserved2;
	((GeeAbstractMultiSetClass *) klass)->reserved3 = gee_abstract_multi_set_real_reserved3;
	((GeeAbstractMultiSetClass *) klass)->reserved4 = gee_abstract_multi_set_real_reserved4;
	((GeeAbstractMultiSetClass *) klass)->reserved5 = gee_abstract_multi_set_real_reserved5;
	((GeeAbstractMultiSetClass *) klass)->reserved6 = gee_abstract_multi_set_real_reserved6;
	((GeeAbstractMultiSetClass *) klass)->reserved7 = gee_abstract_multi_set_real_reserved7;
	((GeeAbstractMultiSetClass *) klass)->reserved8 = gee_abstract_multi_set_real_reserved8;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_size = gee_abstract_multi_set_real_get_size;
	GEE_ABSTRACT_COLLECTION_CLASS (klass)->get_read_only = gee_abstract_multi_set_real_get_read_only;
	GEE_ABSTRACT_MULTI_SET_CLASS (klass)->get_read_only_view = gee_abstract_multi_set_real_get_read_only_view;
	G_OBJECT_CLASS (klass)->get_property = _vala_gee_abstract_multi_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = _vala_gee_abstract_multi_set_set_property;
	G_OBJECT_CLASS (klass)->constructor = gee_abstract_multi_set_constructor;
	G_OBJECT_CLASS (klass)->finalize = gee_abstract_multi_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_TYPE, g_param_spec_gtype ("g-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC, g_param_spec_pointer ("g-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC, g_param_spec_pointer ("g-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_ABSTRACT_MULTI_SET_READ_ONLY, g_param_spec_boolean ("read-only", "read-only", "read-only", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static GType gee_abstract_multi_set_gee_multi_set_get_g_type (GeeAbstractMultiSet* self) {
	return self->priv->g_type;
}


static GBoxedCopyFunc gee_abstract_multi_set_gee_multi_set_get_g_dup_func (GeeAbstractMultiSet* self) {
	return self->priv->g_dup_func;
}


static GDestroyNotify gee_abstract_multi_set_gee_multi_set_get_g_destroy_func (GeeAbstractMultiSet* self) {
	return self->priv->g_destroy_func;
}


static void gee_abstract_multi_set_gee_multi_set_interface_init (GeeMultiSetIface * iface) {
	gee_abstract_multi_set_gee_multi_set_parent_iface = g_type_interface_peek_parent (iface);
	iface->count = (gint (*)(GeeMultiSet*, gconstpointer)) gee_abstract_multi_set_real_count;
	iface->get_g_type = (GType(*)(GeeMultiSet*)) gee_abstract_multi_set_gee_multi_set_get_g_type;
	iface->get_g_dup_func = (GBoxedCopyFunc(*)(GeeMultiSet*)) gee_abstract_multi_set_gee_multi_set_get_g_dup_func;
	iface->get_g_destroy_func = (GDestroyNotify(*)(GeeMultiSet*)) gee_abstract_multi_set_gee_multi_set_get_g_destroy_func;
	iface->get_read_only_view = (GeeMultiSet* (*) (GeeMultiSet *)) gee_abstract_multi_set_get_read_only_view;
}


static void gee_abstract_multi_set_instance_init (GeeAbstractMultiSet * self) {
	self->priv = GEE_ABSTRACT_MULTI_SET_GET_PRIVATE (self);
	self->priv->_nitems = 0;
}


static void gee_abstract_multi_set_finalize (GObject* obj) {
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	_g_object_unref0 (self->_storage_map);
	g_weak_ref_clear (&self->priv->_read_only_view);
	G_OBJECT_CLASS (gee_abstract_multi_set_parent_class)->finalize (obj);
}


/**
 * Skeletal implementation of the {@link MultiSet} interface.
 *
 * @see HashMultiSet
 * @see TreeMultiSet
 */
GType gee_abstract_multi_set_get_type (void) {
	static volatile gsize gee_abstract_multi_set_type_id__volatile = 0;
	if (g_once_init_enter (&gee_abstract_multi_set_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeAbstractMultiSetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_abstract_multi_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeAbstractMultiSet), 0, (GInstanceInitFunc) gee_abstract_multi_set_instance_init, NULL };
		static const GInterfaceInfo gee_multi_set_info = { (GInterfaceInitFunc) gee_abstract_multi_set_gee_multi_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		GType gee_abstract_multi_set_type_id;
		gee_abstract_multi_set_type_id = g_type_register_static (GEE_TYPE_ABSTRACT_COLLECTION, "GeeAbstractMultiSet", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
		g_type_add_interface_static (gee_abstract_multi_set_type_id, GEE_TYPE_MULTI_SET, &gee_multi_set_info);
		g_once_init_leave (&gee_abstract_multi_set_type_id__volatile, gee_abstract_multi_set_type_id);
	}
	return gee_abstract_multi_set_type_id__volatile;
}


static void _vala_gee_abstract_multi_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_SIZE:
		g_value_set_int (value, gee_abstract_collection_get_size ((GeeAbstractCollection*) self));
		break;
		case GEE_ABSTRACT_MULTI_SET_READ_ONLY:
		g_value_set_boolean (value, gee_abstract_collection_get_read_only ((GeeAbstractCollection*) self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_gee_abstract_multi_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeAbstractMultiSet * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (object, GEE_TYPE_ABSTRACT_MULTI_SET, GeeAbstractMultiSet);
	switch (property_id) {
		case GEE_ABSTRACT_MULTI_SET_G_TYPE:
		self->priv->g_type = g_value_get_gtype (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DUP_FUNC:
		self->priv->g_dup_func = g_value_get_pointer (value);
		break;
		case GEE_ABSTRACT_MULTI_SET_G_DESTROY_FUNC:
		self->priv->g_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}



