/* testmain.c generated by valac 0.25.1.16-fa2259, the Vala compiler
 * generated from testmain.vala, do not modify */

/* testmain.vala
 *
 * Copyright (C) 2008  Jürg Billeter
 * Copyright (C) 2009  Didier Villevalois, Julien Peeters
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes' Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>


#define GEE_TYPE_TEST_CASE (gee_test_case_get_type ())
#define GEE_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_TEST_CASE, GeeTestCase))
#define GEE_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_TEST_CASE, GeeTestCaseClass))
#define GEE_IS_TEST_CASE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_TEST_CASE))
#define GEE_IS_TEST_CASE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_TEST_CASE))
#define GEE_TEST_CASE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_TEST_CASE, GeeTestCaseClass))

typedef struct _GeeTestCase GeeTestCase;
typedef struct _GeeTestCaseClass GeeTestCaseClass;

#define TYPE_COLLECTION_TESTS (collection_tests_get_type ())
#define COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_COLLECTION_TESTS, CollectionTests))
#define COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_COLLECTION_TESTS, CollectionTestsClass))
#define IS_COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_COLLECTION_TESTS))
#define IS_COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_COLLECTION_TESTS))
#define COLLECTION_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_COLLECTION_TESTS, CollectionTestsClass))

typedef struct _CollectionTests CollectionTests;
typedef struct _CollectionTestsClass CollectionTestsClass;

#define TYPE_LIST_TESTS (list_tests_get_type ())
#define LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LIST_TESTS, ListTests))
#define LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LIST_TESTS, ListTestsClass))
#define IS_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LIST_TESTS))
#define IS_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LIST_TESTS))
#define LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LIST_TESTS, ListTestsClass))

typedef struct _ListTests ListTests;
typedef struct _ListTestsClass ListTestsClass;

#define TYPE_BIDIR_LIST_TESTS (bidir_list_tests_get_type ())
#define BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BIDIR_LIST_TESTS, BidirListTests))
#define BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BIDIR_LIST_TESTS, BidirListTestsClass))
#define IS_BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BIDIR_LIST_TESTS))
#define IS_BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BIDIR_LIST_TESTS))
#define BIDIR_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BIDIR_LIST_TESTS, BidirListTestsClass))

typedef struct _BidirListTests BidirListTests;
typedef struct _BidirListTestsClass BidirListTestsClass;

#define TYPE_ARRAY_LIST_TESTS (array_list_tests_get_type ())
#define ARRAY_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ARRAY_LIST_TESTS, ArrayListTests))
#define ARRAY_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ARRAY_LIST_TESTS, ArrayListTestsClass))
#define IS_ARRAY_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ARRAY_LIST_TESTS))
#define IS_ARRAY_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ARRAY_LIST_TESTS))
#define ARRAY_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ARRAY_LIST_TESTS, ArrayListTestsClass))

typedef struct _ArrayListTests ArrayListTests;
typedef struct _ArrayListTestsClass ArrayListTestsClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define TYPE_QUEUE_TESTS (queue_tests_get_type ())
#define QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_QUEUE_TESTS, QueueTests))
#define QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_QUEUE_TESTS, QueueTestsClass))
#define IS_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_QUEUE_TESTS))
#define IS_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_QUEUE_TESTS))
#define QUEUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_QUEUE_TESTS, QueueTestsClass))

typedef struct _QueueTests QueueTests;
typedef struct _QueueTestsClass QueueTestsClass;

#define TYPE_DEQUE_TESTS (deque_tests_get_type ())
#define DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_DEQUE_TESTS, DequeTests))
#define DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_DEQUE_TESTS, DequeTestsClass))
#define IS_DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_DEQUE_TESTS))
#define IS_DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_DEQUE_TESTS))
#define DEQUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_DEQUE_TESTS, DequeTestsClass))

typedef struct _DequeTests DequeTests;
typedef struct _DequeTestsClass DequeTestsClass;

#define TYPE_ARRAY_QUEUE_TESTS (array_queue_tests_get_type ())
#define ARRAY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ARRAY_QUEUE_TESTS, ArrayQueueTests))
#define ARRAY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ARRAY_QUEUE_TESTS, ArrayQueueTestsClass))
#define IS_ARRAY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ARRAY_QUEUE_TESTS))
#define IS_ARRAY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ARRAY_QUEUE_TESTS))
#define ARRAY_QUEUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_ARRAY_QUEUE_TESTS, ArrayQueueTestsClass))

typedef struct _ArrayQueueTests ArrayQueueTests;
typedef struct _ArrayQueueTestsClass ArrayQueueTestsClass;

#define TYPE_CONCURRENT_LIST_TESTS (concurrent_list_tests_get_type ())
#define CONCURRENT_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONCURRENT_LIST_TESTS, ConcurrentListTests))
#define CONCURRENT_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONCURRENT_LIST_TESTS, ConcurrentListTestsClass))
#define IS_CONCURRENT_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONCURRENT_LIST_TESTS))
#define IS_CONCURRENT_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONCURRENT_LIST_TESTS))
#define CONCURRENT_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONCURRENT_LIST_TESTS, ConcurrentListTestsClass))

typedef struct _ConcurrentListTests ConcurrentListTests;
typedef struct _ConcurrentListTestsClass ConcurrentListTestsClass;

#define TYPE_SET_TESTS (set_tests_get_type ())
#define SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SET_TESTS, SetTests))
#define SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SET_TESTS, SetTestsClass))
#define IS_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SET_TESTS))
#define IS_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SET_TESTS))
#define SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SET_TESTS, SetTestsClass))

typedef struct _SetTests SetTests;
typedef struct _SetTestsClass SetTestsClass;

#define TYPE_SORTED_SET_TESTS (sorted_set_tests_get_type ())
#define SORTED_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_SORTED_SET_TESTS, SortedSetTests))
#define SORTED_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_SORTED_SET_TESTS, SortedSetTestsClass))
#define IS_SORTED_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_SORTED_SET_TESTS))
#define IS_SORTED_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_SORTED_SET_TESTS))
#define SORTED_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_SORTED_SET_TESTS, SortedSetTestsClass))

typedef struct _SortedSetTests SortedSetTests;
typedef struct _SortedSetTestsClass SortedSetTestsClass;

#define TYPE_CONCURRENT_SET_TESTS (concurrent_set_tests_get_type ())
#define CONCURRENT_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_CONCURRENT_SET_TESTS, ConcurrentSetTests))
#define CONCURRENT_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_CONCURRENT_SET_TESTS, ConcurrentSetTestsClass))
#define IS_CONCURRENT_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_CONCURRENT_SET_TESTS))
#define IS_CONCURRENT_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_CONCURRENT_SET_TESTS))
#define CONCURRENT_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_CONCURRENT_SET_TESTS, ConcurrentSetTestsClass))

typedef struct _ConcurrentSetTests ConcurrentSetTests;
typedef struct _ConcurrentSetTestsClass ConcurrentSetTestsClass;

#define TYPE_FUNCTIONS_TESTS (functions_tests_get_type ())
#define FUNCTIONS_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_FUNCTIONS_TESTS, FunctionsTests))
#define FUNCTIONS_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_FUNCTIONS_TESTS, FunctionsTestsClass))
#define IS_FUNCTIONS_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_FUNCTIONS_TESTS))
#define IS_FUNCTIONS_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_FUNCTIONS_TESTS))
#define FUNCTIONS_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_FUNCTIONS_TESTS, FunctionsTestsClass))

typedef struct _FunctionsTests FunctionsTests;
typedef struct _FunctionsTestsClass FunctionsTestsClass;

#define TYPE_MAP_TESTS (map_tests_get_type ())
#define MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MAP_TESTS, MapTests))
#define MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MAP_TESTS, MapTestsClass))
#define IS_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MAP_TESTS))
#define IS_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MAP_TESTS))
#define MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MAP_TESTS, MapTestsClass))

typedef struct _MapTests MapTests;
typedef struct _MapTestsClass MapTestsClass;

#define TYPE_HASH_MAP_TESTS (hash_map_tests_get_type ())
#define HASH_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_MAP_TESTS, HashMapTests))
#define HASH_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_MAP_TESTS, HashMapTestsClass))
#define IS_HASH_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_MAP_TESTS))
#define IS_HASH_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_MAP_TESTS))
#define HASH_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_MAP_TESTS, HashMapTestsClass))

typedef struct _HashMapTests HashMapTests;
typedef struct _HashMapTestsClass HashMapTestsClass;

#define TYPE_MULTI_MAP_TESTS (multi_map_tests_get_type ())
#define MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MULTI_MAP_TESTS, MultiMapTests))
#define MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MULTI_MAP_TESTS, MultiMapTestsClass))
#define IS_MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MULTI_MAP_TESTS))
#define IS_MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MULTI_MAP_TESTS))
#define MULTI_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MULTI_MAP_TESTS, MultiMapTestsClass))

typedef struct _MultiMapTests MultiMapTests;
typedef struct _MultiMapTestsClass MultiMapTestsClass;

#define TYPE_HASH_MULTI_MAP_TESTS (hash_multi_map_tests_get_type ())
#define HASH_MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_MULTI_MAP_TESTS, HashMultiMapTests))
#define HASH_MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_MULTI_MAP_TESTS, HashMultiMapTestsClass))
#define IS_HASH_MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_MULTI_MAP_TESTS))
#define IS_HASH_MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_MULTI_MAP_TESTS))
#define HASH_MULTI_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_MULTI_MAP_TESTS, HashMultiMapTestsClass))

typedef struct _HashMultiMapTests HashMultiMapTests;
typedef struct _HashMultiMapTestsClass HashMultiMapTestsClass;

#define TYPE_MULTI_SET_TESTS (multi_set_tests_get_type ())
#define MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MULTI_SET_TESTS, MultiSetTests))
#define MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MULTI_SET_TESTS, MultiSetTestsClass))
#define IS_MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MULTI_SET_TESTS))
#define IS_MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MULTI_SET_TESTS))
#define MULTI_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MULTI_SET_TESTS, MultiSetTestsClass))

typedef struct _MultiSetTests MultiSetTests;
typedef struct _MultiSetTestsClass MultiSetTestsClass;

#define TYPE_HASH_MULTI_SET_TESTS (hash_multi_set_tests_get_type ())
#define HASH_MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_MULTI_SET_TESTS, HashMultiSetTests))
#define HASH_MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_MULTI_SET_TESTS, HashMultiSetTestsClass))
#define IS_HASH_MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_MULTI_SET_TESTS))
#define IS_HASH_MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_MULTI_SET_TESTS))
#define HASH_MULTI_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_MULTI_SET_TESTS, HashMultiSetTestsClass))

typedef struct _HashMultiSetTests HashMultiSetTests;
typedef struct _HashMultiSetTestsClass HashMultiSetTestsClass;

#define TYPE_HASH_SET_TESTS (hash_set_tests_get_type ())
#define HASH_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_HASH_SET_TESTS, HashSetTests))
#define HASH_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_HASH_SET_TESTS, HashSetTestsClass))
#define IS_HASH_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_HASH_SET_TESTS))
#define IS_HASH_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_HASH_SET_TESTS))
#define HASH_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_HASH_SET_TESTS, HashSetTestsClass))

typedef struct _HashSetTests HashSetTests;
typedef struct _HashSetTestsClass HashSetTestsClass;

#define TYPE_LINKED_LIST_TESTS (linked_list_tests_get_type ())
#define LINKED_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LINKED_LIST_TESTS, LinkedListTests))
#define LINKED_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LINKED_LIST_TESTS, LinkedListTestsClass))
#define IS_LINKED_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LINKED_LIST_TESTS))
#define IS_LINKED_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LINKED_LIST_TESTS))
#define LINKED_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LINKED_LIST_TESTS, LinkedListTestsClass))

typedef struct _LinkedListTests LinkedListTests;
typedef struct _LinkedListTestsClass LinkedListTestsClass;

#define TYPE_LINKED_LIST_AS_DEQUE_TESTS (linked_list_as_deque_tests_get_type ())
#define LINKED_LIST_AS_DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LINKED_LIST_AS_DEQUE_TESTS, LinkedListAsDequeTests))
#define LINKED_LIST_AS_DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LINKED_LIST_AS_DEQUE_TESTS, LinkedListAsDequeTestsClass))
#define IS_LINKED_LIST_AS_DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LINKED_LIST_AS_DEQUE_TESTS))
#define IS_LINKED_LIST_AS_DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LINKED_LIST_AS_DEQUE_TESTS))
#define LINKED_LIST_AS_DEQUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LINKED_LIST_AS_DEQUE_TESTS, LinkedListAsDequeTestsClass))

typedef struct _LinkedListAsDequeTests LinkedListAsDequeTests;
typedef struct _LinkedListAsDequeTestsClass LinkedListAsDequeTestsClass;

#define TYPE_PRIORITY_QUEUE_TESTS (priority_queue_tests_get_type ())
#define PRIORITY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTests))
#define PRIORITY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTestsClass))
#define IS_PRIORITY_QUEUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_PRIORITY_QUEUE_TESTS))
#define IS_PRIORITY_QUEUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_PRIORITY_QUEUE_TESTS))
#define PRIORITY_QUEUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_PRIORITY_QUEUE_TESTS, PriorityQueueTestsClass))

typedef struct _PriorityQueueTests PriorityQueueTests;
typedef struct _PriorityQueueTestsClass PriorityQueueTestsClass;

#define TYPE_READ_ONLY_COLLECTION_TESTS (read_only_collection_tests_get_type ())
#define READ_ONLY_COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_READ_ONLY_COLLECTION_TESTS, ReadOnlyCollectionTests))
#define READ_ONLY_COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_READ_ONLY_COLLECTION_TESTS, ReadOnlyCollectionTestsClass))
#define IS_READ_ONLY_COLLECTION_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_READ_ONLY_COLLECTION_TESTS))
#define IS_READ_ONLY_COLLECTION_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_READ_ONLY_COLLECTION_TESTS))
#define READ_ONLY_COLLECTION_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_READ_ONLY_COLLECTION_TESTS, ReadOnlyCollectionTestsClass))

typedef struct _ReadOnlyCollectionTests ReadOnlyCollectionTests;
typedef struct _ReadOnlyCollectionTestsClass ReadOnlyCollectionTestsClass;

#define TYPE_READ_ONLY_LIST_TESTS (read_only_list_tests_get_type ())
#define READ_ONLY_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_READ_ONLY_LIST_TESTS, ReadOnlyListTests))
#define READ_ONLY_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_READ_ONLY_LIST_TESTS, ReadOnlyListTestsClass))
#define IS_READ_ONLY_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_READ_ONLY_LIST_TESTS))
#define IS_READ_ONLY_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_READ_ONLY_LIST_TESTS))
#define READ_ONLY_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_READ_ONLY_LIST_TESTS, ReadOnlyListTestsClass))

typedef struct _ReadOnlyListTests ReadOnlyListTests;
typedef struct _ReadOnlyListTestsClass ReadOnlyListTestsClass;

#define TYPE_READ_ONLY_BIDIR_LIST_TESTS (read_only_bidir_list_tests_get_type ())
#define READ_ONLY_BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_READ_ONLY_BIDIR_LIST_TESTS, ReadOnlyBidirListTests))
#define READ_ONLY_BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_READ_ONLY_BIDIR_LIST_TESTS, ReadOnlyBidirListTestsClass))
#define IS_READ_ONLY_BIDIR_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_READ_ONLY_BIDIR_LIST_TESTS))
#define IS_READ_ONLY_BIDIR_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_READ_ONLY_BIDIR_LIST_TESTS))
#define READ_ONLY_BIDIR_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_READ_ONLY_BIDIR_LIST_TESTS, ReadOnlyBidirListTestsClass))

typedef struct _ReadOnlyBidirListTests ReadOnlyBidirListTests;
typedef struct _ReadOnlyBidirListTestsClass ReadOnlyBidirListTestsClass;

#define TYPE_READ_ONLY_MAP_TESTS (read_only_map_tests_get_type ())
#define READ_ONLY_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_READ_ONLY_MAP_TESTS, ReadOnlyMapTests))
#define READ_ONLY_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_READ_ONLY_MAP_TESTS, ReadOnlyMapTestsClass))
#define IS_READ_ONLY_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_READ_ONLY_MAP_TESTS))
#define IS_READ_ONLY_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_READ_ONLY_MAP_TESTS))
#define READ_ONLY_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_READ_ONLY_MAP_TESTS, ReadOnlyMapTestsClass))

typedef struct _ReadOnlyMapTests ReadOnlyMapTests;
typedef struct _ReadOnlyMapTestsClass ReadOnlyMapTestsClass;

#define TYPE_READ_ONLY_SET_TESTS (read_only_set_tests_get_type ())
#define READ_ONLY_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_READ_ONLY_SET_TESTS, ReadOnlySetTests))
#define READ_ONLY_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_READ_ONLY_SET_TESTS, ReadOnlySetTestsClass))
#define IS_READ_ONLY_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_READ_ONLY_SET_TESTS))
#define IS_READ_ONLY_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_READ_ONLY_SET_TESTS))
#define READ_ONLY_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_READ_ONLY_SET_TESTS, ReadOnlySetTestsClass))

typedef struct _ReadOnlySetTests ReadOnlySetTests;
typedef struct _ReadOnlySetTestsClass ReadOnlySetTestsClass;

#define GEE_TYPE_SORTED_MAP_TESTS (gee_sorted_map_tests_get_type ())
#define GEE_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTests))
#define GEE_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTestsClass))
#define GEE_IS_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_SORTED_MAP_TESTS))
#define GEE_IS_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_TYPE_SORTED_MAP_TESTS))
#define GEE_SORTED_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_TYPE_SORTED_MAP_TESTS, GeeSortedMapTestsClass))

typedef struct _GeeSortedMapTests GeeSortedMapTests;
typedef struct _GeeSortedMapTestsClass GeeSortedMapTestsClass;

#define TYPE_BIDIR_SORTED_MAP_TESTS (bidir_sorted_map_tests_get_type ())
#define BIDIR_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTests))
#define BIDIR_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTestsClass))
#define IS_BIDIR_SORTED_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BIDIR_SORTED_MAP_TESTS))
#define IS_BIDIR_SORTED_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BIDIR_SORTED_MAP_TESTS))
#define BIDIR_SORTED_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BIDIR_SORTED_MAP_TESTS, BidirSortedMapTestsClass))

typedef struct _BidirSortedMapTests BidirSortedMapTests;
typedef struct _BidirSortedMapTestsClass BidirSortedMapTestsClass;

#define TYPE_TREE_MAP_TESTS (tree_map_tests_get_type ())
#define TREE_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE_MAP_TESTS, TreeMapTests))
#define TREE_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE_MAP_TESTS, TreeMapTestsClass))
#define IS_TREE_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE_MAP_TESTS))
#define IS_TREE_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE_MAP_TESTS))
#define TREE_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE_MAP_TESTS, TreeMapTestsClass))

typedef struct _TreeMapTests TreeMapTests;
typedef struct _TreeMapTestsClass TreeMapTestsClass;

#define TYPE_TREE_MULTI_MAP_TESTS (tree_multi_map_tests_get_type ())
#define TREE_MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE_MULTI_MAP_TESTS, TreeMultiMapTests))
#define TREE_MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE_MULTI_MAP_TESTS, TreeMultiMapTestsClass))
#define IS_TREE_MULTI_MAP_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE_MULTI_MAP_TESTS))
#define IS_TREE_MULTI_MAP_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE_MULTI_MAP_TESTS))
#define TREE_MULTI_MAP_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE_MULTI_MAP_TESTS, TreeMultiMapTestsClass))

typedef struct _TreeMultiMapTests TreeMultiMapTests;
typedef struct _TreeMultiMapTestsClass TreeMultiMapTestsClass;

#define TYPE_TREE_MULTI_SET_TESTS (tree_multi_set_tests_get_type ())
#define TREE_MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE_MULTI_SET_TESTS, TreeMultiSetTests))
#define TREE_MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE_MULTI_SET_TESTS, TreeMultiSetTestsClass))
#define IS_TREE_MULTI_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE_MULTI_SET_TESTS))
#define IS_TREE_MULTI_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE_MULTI_SET_TESTS))
#define TREE_MULTI_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE_MULTI_SET_TESTS, TreeMultiSetTestsClass))

typedef struct _TreeMultiSetTests TreeMultiSetTests;
typedef struct _TreeMultiSetTestsClass TreeMultiSetTestsClass;

#define TYPE_BIDIR_SORTED_SET_TESTS (bidir_sorted_set_tests_get_type ())
#define BIDIR_SORTED_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_BIDIR_SORTED_SET_TESTS, BidirSortedSetTests))
#define BIDIR_SORTED_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_BIDIR_SORTED_SET_TESTS, BidirSortedSetTestsClass))
#define IS_BIDIR_SORTED_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_BIDIR_SORTED_SET_TESTS))
#define IS_BIDIR_SORTED_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_BIDIR_SORTED_SET_TESTS))
#define BIDIR_SORTED_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_BIDIR_SORTED_SET_TESTS, BidirSortedSetTestsClass))

typedef struct _BidirSortedSetTests BidirSortedSetTests;
typedef struct _BidirSortedSetTestsClass BidirSortedSetTestsClass;

#define TYPE_TREE_SET_TESTS (tree_set_tests_get_type ())
#define TREE_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_TREE_SET_TESTS, TreeSetTests))
#define TREE_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_TREE_SET_TESTS, TreeSetTestsClass))
#define IS_TREE_SET_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_TREE_SET_TESTS))
#define IS_TREE_SET_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_TREE_SET_TESTS))
#define TREE_SET_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_TREE_SET_TESTS, TreeSetTestsClass))

typedef struct _TreeSetTests TreeSetTests;
typedef struct _TreeSetTestsClass TreeSetTestsClass;

#define TYPE_UNROLLED_LINKED_LIST_TESTS (unrolled_linked_list_tests_get_type ())
#define UNROLLED_LINKED_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNROLLED_LINKED_LIST_TESTS, UnrolledLinkedListTests))
#define UNROLLED_LINKED_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNROLLED_LINKED_LIST_TESTS, UnrolledLinkedListTestsClass))
#define IS_UNROLLED_LINKED_LIST_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNROLLED_LINKED_LIST_TESTS))
#define IS_UNROLLED_LINKED_LIST_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNROLLED_LINKED_LIST_TESTS))
#define UNROLLED_LINKED_LIST_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNROLLED_LINKED_LIST_TESTS, UnrolledLinkedListTestsClass))

typedef struct _UnrolledLinkedListTests UnrolledLinkedListTests;
typedef struct _UnrolledLinkedListTestsClass UnrolledLinkedListTestsClass;

#define TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS (unrolled_linked_list_as_deque_tests_get_type ())
#define UNROLLED_LINKED_LIST_AS_DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS, UnrolledLinkedListAsDequeTests))
#define UNROLLED_LINKED_LIST_AS_DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS, UnrolledLinkedListAsDequeTestsClass))
#define IS_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS))
#define IS_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS))
#define UNROLLED_LINKED_LIST_AS_DEQUE_TESTS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_UNROLLED_LINKED_LIST_AS_DEQUE_TESTS, UnrolledLinkedListAsDequeTestsClass))

typedef struct _UnrolledLinkedListAsDequeTests UnrolledLinkedListAsDequeTests;
typedef struct _UnrolledLinkedListAsDequeTestsClass UnrolledLinkedListAsDequeTestsClass;



G_GNUC_INTERNAL void _vala_main (gchar** args, int args_length1);
ArrayListTests* array_list_tests_new (void);
ArrayListTests* array_list_tests_construct (GType object_type);
GType gee_test_case_get_type (void) G_GNUC_CONST;
GType collection_tests_get_type (void) G_GNUC_CONST;
GType list_tests_get_type (void) G_GNUC_CONST;
GType bidir_list_tests_get_type (void) G_GNUC_CONST;
GType array_list_tests_get_type (void) G_GNUC_CONST;
GTestSuite* gee_test_case_get_suite (GeeTestCase* self);
ArrayQueueTests* array_queue_tests_new (void);
ArrayQueueTests* array_queue_tests_construct (GType object_type);
GType queue_tests_get_type (void) G_GNUC_CONST;
GType deque_tests_get_type (void) G_GNUC_CONST;
GType array_queue_tests_get_type (void) G_GNUC_CONST;
ConcurrentListTests* concurrent_list_tests_new (void);
ConcurrentListTests* concurrent_list_tests_construct (GType object_type);
GType concurrent_list_tests_get_type (void) G_GNUC_CONST;
ConcurrentSetTests* concurrent_set_tests_new (void);
ConcurrentSetTests* concurrent_set_tests_construct (GType object_type);
GType set_tests_get_type (void) G_GNUC_CONST;
GType sorted_set_tests_get_type (void) G_GNUC_CONST;
GType concurrent_set_tests_get_type (void) G_GNUC_CONST;
FunctionsTests* functions_tests_new (void);
FunctionsTests* functions_tests_construct (GType object_type);
GType functions_tests_get_type (void) G_GNUC_CONST;
HashMapTests* hash_map_tests_new (void);
HashMapTests* hash_map_tests_construct (GType object_type);
GType map_tests_get_type (void) G_GNUC_CONST;
GType hash_map_tests_get_type (void) G_GNUC_CONST;
HashMultiMapTests* hash_multi_map_tests_new (void);
HashMultiMapTests* hash_multi_map_tests_construct (GType object_type);
GType multi_map_tests_get_type (void) G_GNUC_CONST;
GType hash_multi_map_tests_get_type (void) G_GNUC_CONST;
HashMultiSetTests* hash_multi_set_tests_new (void);
HashMultiSetTests* hash_multi_set_tests_construct (GType object_type);
GType multi_set_tests_get_type (void) G_GNUC_CONST;
GType hash_multi_set_tests_get_type (void) G_GNUC_CONST;
HashSetTests* hash_set_tests_new (void);
HashSetTests* hash_set_tests_construct (GType object_type);
GType hash_set_tests_get_type (void) G_GNUC_CONST;
LinkedListTests* linked_list_tests_new (void);
LinkedListTests* linked_list_tests_construct (GType object_type);
GType linked_list_tests_get_type (void) G_GNUC_CONST;
LinkedListAsDequeTests* linked_list_as_deque_tests_new (void);
LinkedListAsDequeTests* linked_list_as_deque_tests_construct (GType object_type);
GType linked_list_as_deque_tests_get_type (void) G_GNUC_CONST;
PriorityQueueTests* priority_queue_tests_new (void);
PriorityQueueTests* priority_queue_tests_construct (GType object_type);
GType priority_queue_tests_get_type (void) G_GNUC_CONST;
ReadOnlyBidirListTests* read_only_bidir_list_tests_new (void);
ReadOnlyBidirListTests* read_only_bidir_list_tests_construct (GType object_type);
GType read_only_collection_tests_get_type (void) G_GNUC_CONST;
GType read_only_list_tests_get_type (void) G_GNUC_CONST;
GType read_only_bidir_list_tests_get_type (void) G_GNUC_CONST;
ReadOnlyCollectionTests* read_only_collection_tests_new (void);
ReadOnlyCollectionTests* read_only_collection_tests_construct (GType object_type);
ReadOnlyListTests* read_only_list_tests_new (void);
ReadOnlyListTests* read_only_list_tests_construct (GType object_type);
ReadOnlyMapTests* read_only_map_tests_new (void);
ReadOnlyMapTests* read_only_map_tests_construct (GType object_type);
GType read_only_map_tests_get_type (void) G_GNUC_CONST;
ReadOnlySetTests* read_only_set_tests_new (void);
ReadOnlySetTests* read_only_set_tests_construct (GType object_type);
GType read_only_set_tests_get_type (void) G_GNUC_CONST;
TreeMapTests* tree_map_tests_new (void);
TreeMapTests* tree_map_tests_construct (GType object_type);
GType gee_sorted_map_tests_get_type (void) G_GNUC_CONST;
GType bidir_sorted_map_tests_get_type (void) G_GNUC_CONST;
GType tree_map_tests_get_type (void) G_GNUC_CONST;
TreeMultiMapTests* tree_multi_map_tests_new (void);
TreeMultiMapTests* tree_multi_map_tests_construct (GType object_type);
GType tree_multi_map_tests_get_type (void) G_GNUC_CONST;
TreeMultiSetTests* tree_multi_set_tests_new (void);
TreeMultiSetTests* tree_multi_set_tests_construct (GType object_type);
GType tree_multi_set_tests_get_type (void) G_GNUC_CONST;
TreeSetTests* tree_set_tests_new (void);
TreeSetTests* tree_set_tests_construct (GType object_type);
GType bidir_sorted_set_tests_get_type (void) G_GNUC_CONST;
GType tree_set_tests_get_type (void) G_GNUC_CONST;
UnrolledLinkedListTests* unrolled_linked_list_tests_new (void);
UnrolledLinkedListTests* unrolled_linked_list_tests_construct (GType object_type);
GType unrolled_linked_list_tests_get_type (void) G_GNUC_CONST;
UnrolledLinkedListAsDequeTests* unrolled_linked_list_as_deque_tests_new (void);
UnrolledLinkedListAsDequeTests* unrolled_linked_list_as_deque_tests_construct (GType object_type);
GType unrolled_linked_list_as_deque_tests_get_type (void) G_GNUC_CONST;


G_GNUC_INTERNAL void _vala_main (gchar** args, int args_length1) {
	GTestSuite* _tmp0_ = NULL;
	ArrayListTests* _tmp1_ = NULL;
	ArrayListTests* _tmp2_ = NULL;
	GTestSuite* _tmp3_ = NULL;
	GTestSuite* _tmp4_ = NULL;
	ArrayQueueTests* _tmp5_ = NULL;
	ArrayQueueTests* _tmp6_ = NULL;
	GTestSuite* _tmp7_ = NULL;
	GTestSuite* _tmp8_ = NULL;
	ConcurrentListTests* _tmp9_ = NULL;
	ConcurrentListTests* _tmp10_ = NULL;
	GTestSuite* _tmp11_ = NULL;
	GTestSuite* _tmp12_ = NULL;
	ConcurrentSetTests* _tmp13_ = NULL;
	ConcurrentSetTests* _tmp14_ = NULL;
	GTestSuite* _tmp15_ = NULL;
	GTestSuite* _tmp16_ = NULL;
	FunctionsTests* _tmp17_ = NULL;
	FunctionsTests* _tmp18_ = NULL;
	GTestSuite* _tmp19_ = NULL;
	GTestSuite* _tmp20_ = NULL;
	HashMapTests* _tmp21_ = NULL;
	HashMapTests* _tmp22_ = NULL;
	GTestSuite* _tmp23_ = NULL;
	GTestSuite* _tmp24_ = NULL;
	HashMultiMapTests* _tmp25_ = NULL;
	HashMultiMapTests* _tmp26_ = NULL;
	GTestSuite* _tmp27_ = NULL;
	GTestSuite* _tmp28_ = NULL;
	HashMultiSetTests* _tmp29_ = NULL;
	HashMultiSetTests* _tmp30_ = NULL;
	GTestSuite* _tmp31_ = NULL;
	GTestSuite* _tmp32_ = NULL;
	HashSetTests* _tmp33_ = NULL;
	HashSetTests* _tmp34_ = NULL;
	GTestSuite* _tmp35_ = NULL;
	GTestSuite* _tmp36_ = NULL;
	LinkedListTests* _tmp37_ = NULL;
	LinkedListTests* _tmp38_ = NULL;
	GTestSuite* _tmp39_ = NULL;
	GTestSuite* _tmp40_ = NULL;
	LinkedListAsDequeTests* _tmp41_ = NULL;
	LinkedListAsDequeTests* _tmp42_ = NULL;
	GTestSuite* _tmp43_ = NULL;
	GTestSuite* _tmp44_ = NULL;
	PriorityQueueTests* _tmp45_ = NULL;
	PriorityQueueTests* _tmp46_ = NULL;
	GTestSuite* _tmp47_ = NULL;
	GTestSuite* _tmp48_ = NULL;
	ReadOnlyBidirListTests* _tmp49_ = NULL;
	ReadOnlyBidirListTests* _tmp50_ = NULL;
	GTestSuite* _tmp51_ = NULL;
	GTestSuite* _tmp52_ = NULL;
	ReadOnlyCollectionTests* _tmp53_ = NULL;
	ReadOnlyCollectionTests* _tmp54_ = NULL;
	GTestSuite* _tmp55_ = NULL;
	GTestSuite* _tmp56_ = NULL;
	ReadOnlyListTests* _tmp57_ = NULL;
	ReadOnlyListTests* _tmp58_ = NULL;
	GTestSuite* _tmp59_ = NULL;
	GTestSuite* _tmp60_ = NULL;
	ReadOnlyMapTests* _tmp61_ = NULL;
	ReadOnlyMapTests* _tmp62_ = NULL;
	GTestSuite* _tmp63_ = NULL;
	GTestSuite* _tmp64_ = NULL;
	ReadOnlySetTests* _tmp65_ = NULL;
	ReadOnlySetTests* _tmp66_ = NULL;
	GTestSuite* _tmp67_ = NULL;
	GTestSuite* _tmp68_ = NULL;
	TreeMapTests* _tmp69_ = NULL;
	TreeMapTests* _tmp70_ = NULL;
	GTestSuite* _tmp71_ = NULL;
	GTestSuite* _tmp72_ = NULL;
	TreeMultiMapTests* _tmp73_ = NULL;
	TreeMultiMapTests* _tmp74_ = NULL;
	GTestSuite* _tmp75_ = NULL;
	GTestSuite* _tmp76_ = NULL;
	TreeMultiSetTests* _tmp77_ = NULL;
	TreeMultiSetTests* _tmp78_ = NULL;
	GTestSuite* _tmp79_ = NULL;
	GTestSuite* _tmp80_ = NULL;
	TreeSetTests* _tmp81_ = NULL;
	TreeSetTests* _tmp82_ = NULL;
	GTestSuite* _tmp83_ = NULL;
	GTestSuite* _tmp84_ = NULL;
	UnrolledLinkedListTests* _tmp85_ = NULL;
	UnrolledLinkedListTests* _tmp86_ = NULL;
	GTestSuite* _tmp87_ = NULL;
	GTestSuite* _tmp88_ = NULL;
	UnrolledLinkedListAsDequeTests* _tmp89_ = NULL;
	UnrolledLinkedListAsDequeTests* _tmp90_ = NULL;
	GTestSuite* _tmp91_ = NULL;
	g_test_init (&args_length1, &args, NULL);
	gee_hazard_pointer_set_release_policy (GEE_HAZARD_POINTER_RELEASE_POLICY_MAIN_LOOP);
	_tmp0_ = g_test_get_root ();
	_tmp1_ = array_list_tests_new ();
	_tmp2_ = _tmp1_;
	_tmp3_ = gee_test_case_get_suite ((GeeTestCase*) _tmp2_);
	g_test_suite_add_suite (_tmp0_, _tmp3_);
	_g_object_unref0 (_tmp2_);
	_tmp4_ = g_test_get_root ();
	_tmp5_ = array_queue_tests_new ();
	_tmp6_ = _tmp5_;
	_tmp7_ = gee_test_case_get_suite ((GeeTestCase*) _tmp6_);
	g_test_suite_add_suite (_tmp4_, _tmp7_);
	_g_object_unref0 (_tmp6_);
	_tmp8_ = g_test_get_root ();
	_tmp9_ = concurrent_list_tests_new ();
	_tmp10_ = _tmp9_;
	_tmp11_ = gee_test_case_get_suite ((GeeTestCase*) _tmp10_);
	g_test_suite_add_suite (_tmp8_, _tmp11_);
	_g_object_unref0 (_tmp10_);
	_tmp12_ = g_test_get_root ();
	_tmp13_ = concurrent_set_tests_new ();
	_tmp14_ = _tmp13_;
	_tmp15_ = gee_test_case_get_suite ((GeeTestCase*) _tmp14_);
	g_test_suite_add_suite (_tmp12_, _tmp15_);
	_g_object_unref0 (_tmp14_);
	_tmp16_ = g_test_get_root ();
	_tmp17_ = functions_tests_new ();
	_tmp18_ = _tmp17_;
	_tmp19_ = gee_test_case_get_suite ((GeeTestCase*) _tmp18_);
	g_test_suite_add_suite (_tmp16_, _tmp19_);
	_g_object_unref0 (_tmp18_);
	_tmp20_ = g_test_get_root ();
	_tmp21_ = hash_map_tests_new ();
	_tmp22_ = _tmp21_;
	_tmp23_ = gee_test_case_get_suite ((GeeTestCase*) _tmp22_);
	g_test_suite_add_suite (_tmp20_, _tmp23_);
	_g_object_unref0 (_tmp22_);
	_tmp24_ = g_test_get_root ();
	_tmp25_ = hash_multi_map_tests_new ();
	_tmp26_ = _tmp25_;
	_tmp27_ = gee_test_case_get_suite ((GeeTestCase*) _tmp26_);
	g_test_suite_add_suite (_tmp24_, _tmp27_);
	_g_object_unref0 (_tmp26_);
	_tmp28_ = g_test_get_root ();
	_tmp29_ = hash_multi_set_tests_new ();
	_tmp30_ = _tmp29_;
	_tmp31_ = gee_test_case_get_suite ((GeeTestCase*) _tmp30_);
	g_test_suite_add_suite (_tmp28_, _tmp31_);
	_g_object_unref0 (_tmp30_);
	_tmp32_ = g_test_get_root ();
	_tmp33_ = hash_set_tests_new ();
	_tmp34_ = _tmp33_;
	_tmp35_ = gee_test_case_get_suite ((GeeTestCase*) _tmp34_);
	g_test_suite_add_suite (_tmp32_, _tmp35_);
	_g_object_unref0 (_tmp34_);
	_tmp36_ = g_test_get_root ();
	_tmp37_ = linked_list_tests_new ();
	_tmp38_ = _tmp37_;
	_tmp39_ = gee_test_case_get_suite ((GeeTestCase*) _tmp38_);
	g_test_suite_add_suite (_tmp36_, _tmp39_);
	_g_object_unref0 (_tmp38_);
	_tmp40_ = g_test_get_root ();
	_tmp41_ = linked_list_as_deque_tests_new ();
	_tmp42_ = _tmp41_;
	_tmp43_ = gee_test_case_get_suite ((GeeTestCase*) _tmp42_);
	g_test_suite_add_suite (_tmp40_, _tmp43_);
	_g_object_unref0 (_tmp42_);
	_tmp44_ = g_test_get_root ();
	_tmp45_ = priority_queue_tests_new ();
	_tmp46_ = _tmp45_;
	_tmp47_ = gee_test_case_get_suite ((GeeTestCase*) _tmp46_);
	g_test_suite_add_suite (_tmp44_, _tmp47_);
	_g_object_unref0 (_tmp46_);
	_tmp48_ = g_test_get_root ();
	_tmp49_ = read_only_bidir_list_tests_new ();
	_tmp50_ = _tmp49_;
	_tmp51_ = gee_test_case_get_suite ((GeeTestCase*) _tmp50_);
	g_test_suite_add_suite (_tmp48_, _tmp51_);
	_g_object_unref0 (_tmp50_);
	_tmp52_ = g_test_get_root ();
	_tmp53_ = read_only_collection_tests_new ();
	_tmp54_ = _tmp53_;
	_tmp55_ = gee_test_case_get_suite ((GeeTestCase*) _tmp54_);
	g_test_suite_add_suite (_tmp52_, _tmp55_);
	_g_object_unref0 (_tmp54_);
	_tmp56_ = g_test_get_root ();
	_tmp57_ = read_only_list_tests_new ();
	_tmp58_ = _tmp57_;
	_tmp59_ = gee_test_case_get_suite ((GeeTestCase*) _tmp58_);
	g_test_suite_add_suite (_tmp56_, _tmp59_);
	_g_object_unref0 (_tmp58_);
	_tmp60_ = g_test_get_root ();
	_tmp61_ = read_only_map_tests_new ();
	_tmp62_ = _tmp61_;
	_tmp63_ = gee_test_case_get_suite ((GeeTestCase*) _tmp62_);
	g_test_suite_add_suite (_tmp60_, _tmp63_);
	_g_object_unref0 (_tmp62_);
	_tmp64_ = g_test_get_root ();
	_tmp65_ = read_only_set_tests_new ();
	_tmp66_ = _tmp65_;
	_tmp67_ = gee_test_case_get_suite ((GeeTestCase*) _tmp66_);
	g_test_suite_add_suite (_tmp64_, _tmp67_);
	_g_object_unref0 (_tmp66_);
	_tmp68_ = g_test_get_root ();
	_tmp69_ = tree_map_tests_new ();
	_tmp70_ = _tmp69_;
	_tmp71_ = gee_test_case_get_suite ((GeeTestCase*) _tmp70_);
	g_test_suite_add_suite (_tmp68_, _tmp71_);
	_g_object_unref0 (_tmp70_);
	_tmp72_ = g_test_get_root ();
	_tmp73_ = tree_multi_map_tests_new ();
	_tmp74_ = _tmp73_;
	_tmp75_ = gee_test_case_get_suite ((GeeTestCase*) _tmp74_);
	g_test_suite_add_suite (_tmp72_, _tmp75_);
	_g_object_unref0 (_tmp74_);
	_tmp76_ = g_test_get_root ();
	_tmp77_ = tree_multi_set_tests_new ();
	_tmp78_ = _tmp77_;
	_tmp79_ = gee_test_case_get_suite ((GeeTestCase*) _tmp78_);
	g_test_suite_add_suite (_tmp76_, _tmp79_);
	_g_object_unref0 (_tmp78_);
	_tmp80_ = g_test_get_root ();
	_tmp81_ = tree_set_tests_new ();
	_tmp82_ = _tmp81_;
	_tmp83_ = gee_test_case_get_suite ((GeeTestCase*) _tmp82_);
	g_test_suite_add_suite (_tmp80_, _tmp83_);
	_g_object_unref0 (_tmp82_);
	_tmp84_ = g_test_get_root ();
	_tmp85_ = unrolled_linked_list_tests_new ();
	_tmp86_ = _tmp85_;
	_tmp87_ = gee_test_case_get_suite ((GeeTestCase*) _tmp86_);
	g_test_suite_add_suite (_tmp84_, _tmp87_);
	_g_object_unref0 (_tmp86_);
	_tmp88_ = g_test_get_root ();
	_tmp89_ = unrolled_linked_list_as_deque_tests_new ();
	_tmp90_ = _tmp89_;
	_tmp91_ = gee_test_case_get_suite ((GeeTestCase*) _tmp90_);
	g_test_suite_add_suite (_tmp88_, _tmp91_);
	_g_object_unref0 (_tmp90_);
	g_test_run ();
}


int main (int argc, char ** argv) {
#if !GLIB_CHECK_VERSION (2,35,0)
	g_type_init ();
#endif
	_vala_main (argv, argc);
	return 0;
}



