/* bidirmapiterator.c generated by valac 0.25.3.11-62b1e, the Vala compiler
 * generated from bidirmapiterator.vala, do not modify */

/* bidiriterator.vala
 *
 * Copyright (C) 2009  Didier Villevalois, Maciej Piechotka
 * Copyright (C) 2011  Maciej Piechotka
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Maciej Piechotka <uzytkownik2@gmail.com>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_MAP_ITERATOR (gee_map_iterator_get_type ())
#define GEE_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIterator))
#define GEE_IS_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_MAP_ITERATOR))
#define GEE_MAP_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_MAP_ITERATOR, GeeMapIteratorIface))

typedef struct _GeeMapIterator GeeMapIterator;
typedef struct _GeeMapIteratorIface GeeMapIteratorIface;

#define GEE_TYPE_BIDIR_MAP_ITERATOR (gee_bidir_map_iterator_get_type ())
#define GEE_BIDIR_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIterator))
#define GEE_IS_BIDIR_MAP_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_BIDIR_MAP_ITERATOR))
#define GEE_BIDIR_MAP_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_BIDIR_MAP_ITERATOR, GeeBidirMapIteratorIface))

typedef struct _GeeBidirMapIterator GeeBidirMapIterator;
typedef struct _GeeBidirMapIteratorIface GeeBidirMapIteratorIface;

typedef gpointer (*GeeFoldMapFunc) (gconstpointer k, gconstpointer v, gpointer a, void* user_data);
typedef gboolean (*GeeForallMapFunc) (gconstpointer k, gconstpointer v, void* user_data);
struct _GeeMapIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_k_destroy_func) (GeeMapIterator* self);
	GType (*get_v_type) (GeeMapIterator* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeMapIterator* self);
	GDestroyNotify (*get_v_destroy_func) (GeeMapIterator* self);
	gboolean (*next) (GeeMapIterator* self);
	gboolean (*has_next) (GeeMapIterator* self);
	gpointer (*get_key) (GeeMapIterator* self);
	gpointer (*get_value) (GeeMapIterator* self);
	void (*set_value) (GeeMapIterator* self, gconstpointer value);
	void (*unset) (GeeMapIterator* self);
	gpointer (*fold) (GeeMapIterator* self, GType a_type, GBoxedCopyFunc a_dup_func, GDestroyNotify a_destroy_func, GeeFoldMapFunc f, void* f_target, gpointer seed);
	gboolean (*foreach) (GeeMapIterator* self, GeeForallMapFunc f, void* f_target);
	gboolean (*get_valid) (GeeMapIterator* self);
	gboolean (*get_mutable) (GeeMapIterator* self);
	gboolean (*get_read_only) (GeeMapIterator* self);
};

struct _GeeBidirMapIteratorIface {
	GTypeInterface parent_iface;
	GType (*get_k_type) (GeeBidirMapIterator* self);
	GBoxedCopyFunc (*get_k_dup_func) (GeeBidirMapIterator* self);
	GDestroyNotify (*get_k_destroy_func) (GeeBidirMapIterator* self);
	GType (*get_v_type) (GeeBidirMapIterator* self);
	GBoxedCopyFunc (*get_v_dup_func) (GeeBidirMapIterator* self);
	GDestroyNotify (*get_v_destroy_func) (GeeBidirMapIterator* self);
	gboolean (*previous) (GeeBidirMapIterator* self);
	gboolean (*has_previous) (GeeBidirMapIterator* self);
	gboolean (*first) (GeeBidirMapIterator* self);
	gboolean (*last) (GeeBidirMapIterator* self);
};



GType gee_map_iterator_get_type (void) G_GNUC_CONST;
GType gee_bidir_map_iterator_get_type (void) G_GNUC_CONST;
gboolean gee_bidir_map_iterator_previous (GeeBidirMapIterator* self);
gboolean gee_bidir_map_iterator_has_previous (GeeBidirMapIterator* self);
gboolean gee_bidir_map_iterator_first (GeeBidirMapIterator* self);
gboolean gee_bidir_map_iterator_last (GeeBidirMapIterator* self);


/**
 * Rewinds to the previous element in the iteration.
 *
 * @return `true` if the iterator has a previous element
 */
gboolean gee_bidir_map_iterator_previous (GeeBidirMapIterator* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_BIDIR_MAP_ITERATOR_GET_INTERFACE (self)->previous (self);
}


/**
 * Checks whether there is a previous element in the iteration.
 *
 * @return `true` if the iterator has a previous element
 */
gboolean gee_bidir_map_iterator_has_previous (GeeBidirMapIterator* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_BIDIR_MAP_ITERATOR_GET_INTERFACE (self)->has_previous (self);
}


/**
 * Goes back to the first element.
 *
 * @return `true` if the iterator has a first element
 */
gboolean gee_bidir_map_iterator_first (GeeBidirMapIterator* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_BIDIR_MAP_ITERATOR_GET_INTERFACE (self)->first (self);
}


/**
 * Advances to the last element in the iteration.
 *
 * @return `true` if the iterator has a last element
 */
gboolean gee_bidir_map_iterator_last (GeeBidirMapIterator* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return GEE_BIDIR_MAP_ITERATOR_GET_INTERFACE (self)->last (self);
}


static void gee_bidir_map_iterator_base_init (GeeBidirMapIteratorIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


/**
 * A bi-directional Map iterator.
 */
GType gee_bidir_map_iterator_get_type (void) {
	static volatile gsize gee_bidir_map_iterator_type_id__volatile = 0;
	if (g_once_init_enter (&gee_bidir_map_iterator_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeBidirMapIteratorIface), (GBaseInitFunc) gee_bidir_map_iterator_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType gee_bidir_map_iterator_type_id;
		gee_bidir_map_iterator_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeBidirMapIterator", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gee_bidir_map_iterator_type_id, GEE_TYPE_MAP_ITERATOR);
		g_once_init_leave (&gee_bidir_map_iterator_type_id__volatile, gee_bidir_map_iterator_type_id);
	}
	return gee_bidir_map_iterator_type_id__volatile;
}



